package com.sweep15.GameElements;
 
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
 
/**
 * Classe que implementa una llista ordenada de cartes
 * 
 * @author Sergi Collado Figueras : PFC - Xarxes de computadors 2011-2012
 *
 */
public class CardList
	implements Serializable
{
	private static final long serialVersionUID = 1L;
	
	protected ArrayList<Card> cards;
 
	/* Constructors */ 
	
	public CardList(int n) {
     this.cards = new ArrayList<Card>(n);
	}
 
	public CardList() {
		this.cards = new ArrayList<Card>();
	}
 
	public CardList(CardList cl) {
		this.cards = new ArrayList<Card>(cl.cards.subList(0, cl.cards.size()));
	}
 
	public CardList(List<Card> l) {
		this.cards = new ArrayList<Card>(l);
	}
 
	/* Mtodes */
   
	/**
	 * Afegir carta
	 * 
	 * @param pCard : Carta a afegir
	 */
	public boolean add(Card pCard) {
		this.cards.add(pCard);
		return true;
	}
 
	/**
	 * Afegir carta en posici
	 * 
	 * @param pPos : Posici de la llista on afegir la carta
	 * @param pCard : Carta a afegir
	 * @return true en cas d'afegir-la amb xit
	 */
	public boolean add(int pPos, Card pCard) {
		if ((pPos < 0) || (pPos > this.cards.size())) 
			return false;
		this.cards.add(pPos, pCard);
		return true;
	}
 
	/**
	 * Afegir totes les cartes d'una altra CardList
	 * 
	 * @param pCardList : CardList d'on es volen afegir les cartes
	 */
	public void addAll(CardList pCardList) {
		Iterator<Card> i = pCardList.getIterator();
		while (i.hasNext())
			add((Card)i.next());		
	}
 
	/**
	 * Eliminar la carta de la llista per la seva posici
	 * 
	 * @param pPos : Posici de la carta a eliminar
	 * @return Carta eliminada, o null si no s'ha pogut eliminar
	 */
	public Card remove(int pPos) {
		if ((pPos < 0) || (pPos >= this.cards.size())) 
			return null;
		Card c = (Card)this.cards.remove(pPos);
		return c;
	}
 
	/**
	 * Eliminar una carta de la llista
	 * 
	 * @param pCard : Carta a eliminar
	 * @return Carta eliminada, o null si no s'ha pogut eliminar
	 */
	public Card remove(Card pCard) {
		int i = this.cards.indexOf(pCard);
		Card removedcard = null;
		if (i >= 0) {
			removedcard = (Card)this.cards.remove(i);
		}
		return removedcard;
	}
 
	/**
	 * Eliminar totes les cartes de la llista
	 */
	public void clear() {
		while (remove(0) != null);
	}
 
	/**
	 * Obtenir l'ndex d'una carta en la llista
	 * 
	 * @param pCard : Carta de la que es vol obtenir l'ndex
	 * @return ndex de la carta
	 */
	public int indexOf(Card pCard) {
		return this.cards.indexOf(pCard);
	}
 
	/**
	 * Saber si la llista cont una carta concreta
	 * 
	 * @param pCard : Carta a buscar
	 * @return true si la llista cont la carta
	 */
	public boolean contains(Card pCard) {
		return this.cards.contains(pCard);
	}
  
	/**
	 * Obtenir una carta de la llista per la seva posici
	 * 
	 * @param pPos : Posici de la carta en la llista
	 * @return Carta de la llista, o null si no en troba cap 
	 */
	public Card get(int pPos) {
		if ((pPos < 0) || (pPos >= this.cards.size())) 
			return null;
		return (Card)this.cards.get(pPos);
	}
 
	/**
	 * Obtenir el nombre de cartes de la llista
	 * 
	 * @return nombre de cartes
	 */
	public int size() {
		return this.cards.size();
	}
   
	/**
	 * Saber si la llista est buida
	 * 
	 * @return true en cas que estigui buida
	 */
	public boolean isEmpty() {
		return this.cards.isEmpty();
	}
 
	/**
	 * Obtenir un iterador de tipus ListIterator<Card> per poder recrrer la llista
	 * 
	 * @return iterador de la llista
	 */
	public ListIterator<Card> getIterator() {
		return this.cards.listIterator();
	}
   
	/**
	 * Saber si hi ha alguna animaci (o gir) en el sprite d'alguna de les cartes de la llista
	 * 
	 * @return true en cas afirmatiu
	 */
	public boolean isRunningAnimation() {
		Iterator<Card> iteratorCards = this.cards.iterator();
	    while (iteratorCards.hasNext()) {
	    	Card lCard = (Card)iteratorCards.next();
	        if ((lCard.getSprite().isAnimate()) || (lCard.getSprite().isFlip()))
	        	return true;
	    }
	    return false;
	}		
   
	/**
	 * Obt la suma del valor (rang) de les cartes.
	 * @return Suma del valor
	 */
	public int getRankSum() {
		int rankSum = 0;
		Iterator<Card> i = this.cards.iterator();
	     	while (i.hasNext()) {
	     		Card c = (Card)i.next();
	     		rankSum += c.rank.rankNumber;
	     	}
	     	return rankSum;
	}
   
	/**
	 * Desselecciona tots els sprites de totes les cartes de la llista
	 */
	public void unSelectAll() {	   
		Iterator<Card> i = this.cards.iterator();
	     	while (i.hasNext()) {
	     		Card c = (Card)i.next();
	     		c.getSprite().setSelected(false);
	     	}	     
	}		
 }
