 package com.sweep15.GameElements;

import java.util.Iterator;
 
 /**
  * Classe que implementa la baralla de joc fsica, definida per les cartes de EscobaPackage
  * 
  * @author Sergi Collado Figueras : PFC - Xarxes de computadors 2011-2012
  *
  */
 public class Deck
 {
	 /* Propietats */
	 protected EscobaPackage pkg;
	 protected CardList cardListDeck;
   
	 /* Constructor */
	 
	 public Deck(EscobaPackage pkg) {
		 /* Creem la llista de cartes a partir de les carregades en el EscobaPAckage */
		 this.pkg = pkg;
		 Card[] cards = pkg.getAllCards();
		 this.cardListDeck = new CardList(cards.length);
		 for (int i = 0; i < cards.length; i++)
			 this.cardListDeck.add(cards[i].copyCard());
		 //En segents rondes necessitem ressetejar els indicadors
		 Iterator<Card> iter = this.cardListDeck.getIterator();
		 while (iter.hasNext()) {
			 Card lCard = iter.next();
			 lCard.getSprite().ResetCard();
			 lCard.getSprite().setPosition(0F, 0F);
		 }
		 //Barrejem la baralla
		 this.shuffle();
	 }
 
	 /**
	  * Barreja les cartes de la llista de cartes que conformen la baralla
	  */
	 public void shuffle() {
		 int numCards = this.cardListDeck.size();
		 for (int i = 0; i < numCards * 2; i++) {
			 Card c = extractRandomCard();
			 if (c == null) 
				 continue; 
			 this.cardListDeck.add((int)(Math.random() * numCards), c);
		 }
	 }
 
	 /**
	  * Extreu una carta aleatria de la baralla
	  * 
	  * @return La carta extreta
	  */
	 public Card extractRandomCard() {
		 int numCards = this.cardListDeck.size();
		 if (numCards == 0) return null;
		 int i = (int)(Math.random() * numCards);
		 return this.cardListDeck.remove(i);
	 }
 
	 /**
	  * Extreu vries cartes aleatries de la baralla
	  * 
	  * @param n : nombre de cartes a extreure
	  * @return CardList amb les cartes extretes
	  */
	 public CardList extractRandomCards(int n) {
		 int numCards = this.cardListDeck.size();
		 if (numCards < n) 
			 return null;
		 CardList cardList = new CardList(n);
		 for (int idx = 0; idx < n; idx++) {
			 numCards = this.cardListDeck.size();
			 int i = (int)(Math.random() * numCards);
			 cardList.add(this.cardListDeck.remove(i));
		 }
		 return cardList;
	 }
 
	 /**
	  * Extreu la segent carta de la baralla
	  * 
	  * @return Carta extreta
	  */
	 public Card extractNextCard() {
		 int numCards = this.cardListDeck.size();
		 if (numCards == 0) 
			 return null;
		 return this.cardListDeck.remove(0);
	 }
 
	 /**
	  * Retorna una carta a la baralla
	  * 
	  * @param pCard : carta a retornar
	  */
	 public void returnCard(Card pCard) {
		 this.cardListDeck.add(pCard);
	 }
 
	 /**
	  * Retorna una carta a una posici de la baralla
	  * 
	  * @param pCard : carta a retornar
	  * @param pPos : posici
	  */
	 public void returnCard(Card pCard, int pPos) {
		 this.cardListDeck.add(pPos, pCard);
	 }
 
	 /**
	  * Elimina una carta concreta de la baralla
	  * 
	  * @param pCard : carta a eliminar
	  * @return true si s'ha eliminat satisfactriament
	  */
	 public boolean removeCard(Card pCard) {
		 if (this.cardListDeck.contains(pCard)) {
			 this.cardListDeck.remove(pCard);
			 return true;
		 }
		 return false;
	 }
 
	 /**
	  * Retorna la mida actual de la baralla
	  * 
	  * @return mida de la baralla
	  */
	 public int deckSize() {
		 return this.cardListDeck.size();
	 }
 }