package com.sweep15.GameElements;

import javax.microedition.khronos.opengles.GL10;
import org.anddev.andengine.audio.sound.Sound;
import org.anddev.andengine.engine.camera.Camera;
import org.anddev.andengine.entity.sprite.Sprite;
import org.anddev.andengine.opengl.texture.region.TextureRegion;

/**
 * Classe que implementa l'element Sprite d'una carta (Card)
 * 
 * @author Sergi Collado Figueras : PFC - Xarxes de computadors 2011-2012
 *
 */
public class SpriteCard extends Sprite
{
	/* Constants */
	
	public static final int CARD_WIDTH = 90;
	public static final int CARD_HEIGHT = 126;

	/* Propietats */
	
	private int _cardHeight;
	private int _cardWidth;	
	
	private float _newPosX;
    private float _newPosY;
    private float _posTextureX;
    private float _posTextureY;
    private int _posBackTextureX;
    private int _posBackTextureY;
    private float _positionDeltaX;
    private float _positionDeltaY;
    private float _positionSourceX;
    private float _positionSourceY;
    
    //indicadors
    private boolean _selected;
    private boolean _animate; 
	private boolean _activateFlipSound;
	private boolean _flip;
    private boolean _flipAndAnimate;
    private boolean _inverseFlip;
    
    //Sprites i sons
    private Sprite _selectedCard;
    private Sound _mCardFlipSound;
	
	//clcul de l'efecte al seleccionar
    private float _alpha;
    private boolean _alphaMinor;
    private boolean _alphaPlus;       
    private float _duration;
    private float _elapsed;
    
    private float _percent;
    
    
    /* Constructor */

    public SpriteCard(float pX, float pY, TextureRegion pTextureRegion, Sound pCardFlipSound, Sprite pSelectedCardSprite)
    {
        super(pX, pY, pTextureRegion);
        this._cardWidth = CARD_WIDTH;
        this._cardHeight = CARD_HEIGHT;
        this._newPosX = pX;
        this._newPosY = pY;

        this._animate = false;
        this._flip = false;
        this._percent = 0F;
        this.setSelected(false);        
        this._percent = 0F;
        this._elapsed = 0F;
        this._mCardFlipSound = pCardFlipSound;
        this._activateFlipSound = false;
        this._flipAndAnimate = false;
        this.setInverseFlip(false);
        this._selectedCard = pSelectedCardSprite;
        this._alphaPlus = false;
        this._alphaMinor = true;
        this._alpha = 1F;
        this._posTextureX = pTextureRegion.getTexturePositionX();
        this._posTextureY = pTextureRegion.getTexturePositionY();
        this._posBackTextureX = 8 * this._cardWidth;
        this._posBackTextureY = 1 + 3 * this._cardHeight;
        //AndEngine no deixa cambiar el TextureRegion un cop definit, s per aix que juguem amb la posici per tal
        //de mostrar la carta o el revers
        pTextureRegion.setTexturePosition(this._posBackTextureX, this._posBackTextureY);
    }
    
    /*
     * FUNCIONS pel clcul de moviment i de gir de la carta, segons els exemples trobats al frum de AndEngine
     * i en exemples diversos d'internet.
     */

    private void AccumulateElapsedTime(float time) {
    	this._elapsed = time + this._elapsed;
    }

    private void Transition(float time) {
        this.AccumulateElapsedTime(time);
        if(this._elapsed > this._duration) {
        	this.mX = this._newPosX;
        	this.mY = this._newPosY;
            this._animate = false;
            this._percent = 0F;
            this._elapsed = 0F;
            if(this._flipAndAnimate)
            	this._flipAndAnimate = false;
        } else {
        	this._percent = this._percent + time / this._duration;
        	this.mX = this._positionSourceX + this._positionDeltaX * this._percent;
        	this.mY = this._positionSourceY + this._positionDeltaY * this._percent;
        }
    }

    private void UpdateFlip(float time) {
        AccumulateElapsedTime(time);
        if(this._elapsed > this._duration) {
        	this._flip = false;
        	this._elapsed = 0F;
        	this._percent = 0F;
        	this.setScale(1F, 1F);
        	this._activateFlipSound = false;
        	this.getTextureRegion().setTexturePosition((int)this._posTextureX, (int)this._posTextureY);
        } else {
        	this._percent = this._percent + 100F * (time / this._duration);
            if(this._percent >= 100F)
            	this._percent = 0F;
            if(this._percent < 50F) {
            	this.setScaleX(1F - 2F * (this._percent / 100F));
            } else {
            	this.setScale(2F * (this._percent / 100F) - 1F, 1F);
            	this.getTextureRegion().setTexturePosition((int)this._posTextureX, (int)this._posTextureY);
            }
        }
    }

    private void UpdateFlipInverse(float time) {
    	this.AccumulateElapsedTime(time);
        if(this._elapsed > this._duration) {
        	this.setInverseFlip(false);
        	this._elapsed = 0F;
        	this._percent = 0F;
        	this.setScale(1F, 1F);
        	this._activateFlipSound = false;
        	this.getTextureRegion().setTexturePosition(this._posBackTextureX, this._posBackTextureY);
        } else
        {
        	this._percent = this._percent + 100F * (time / this._duration);
            if(this._percent >= 100F)
            	this._percent = 0F;
            if(this._percent < 50F) {
            	this.setScaleX(1F - 2F * (this._percent / 100F));
            } else {
            	this.setScale(2F * (this._percent / 100F) - 1F, 1F);
            	this.getTextureRegion().setTexturePosition(this._posBackTextureX, this._posBackTextureY);
            }
        }
    }

    /* Mtodes */
    
    /**
     * Reinicialitza els valors
     */
    public void ResetCard() {
    	this._animate = false;
    	this._flip = false;
    	this._percent = 0F;
    	this.setSelected(false);    	
    	this._percent = 0F;
    	this._elapsed = 0F;
    	this._activateFlipSound = false;
    	this._flipAndAnimate = false;
    	this.setInverseFlip(false);
    	this._alphaPlus = false;
    	this._alphaMinor = true;
    	this._alpha = 1F;
    	this.setScale(1F, 1F);
    	this.getTextureRegion().setTexturePosition(this._posBackTextureX, this._posBackTextureY);
    }

    /**
     * Inicia un moviment de transici i de gir de la carta
     * 
     * @param pToX : X de destinaci
     * @param pToY : Y de destinaci
     * @param pDuration : Durada
     */
    public void StartFlipAndAnimate(float pToX, float pToY, float pDuration) {
    	this._flipAndAnimate = true;
    	this._flip = true;
        StartTransition(pToX, pToY, pDuration);
    }

    /**
     * Inicia un moviment de transici de la carta
     * 
     * @param pToX : X de destinaci
     * @param pToY : Y de destinaci
     * @param pDuration : Durada
     */
    public void StartTransition(float pToX, float pToY, float pDuration) {
    	this._duration = pDuration;
    	this._newPosX = pToX;
    	this._newPosY = pToY;
    	this._positionSourceX = mX;
    	this._positionSourceY = mY;
    	this._positionDeltaX = this._newPosX - this._positionSourceX;
    	this._positionDeltaY = this._newPosY - this._positionSourceY;
    	this._animate = true;
    }    

    /**
     * Event que gestiona el dibuixar la carta (segons exemples d'AndEngine)
     */
    protected void onManagedDraw(GL10 gl10, Camera camera) {
        if(this.getSelected()) {
        	//Si la carta est seleccionada, dibuixem el nostre sprite de selecci
        	this._selectedCard.setPosition(getX() - 10F, getY() - 10F);
        	this._selectedCard.onDraw(gl10, camera);
        }
        super.onManagedDraw(gl10, camera);
    }

    /**
     * Event que gestiona el thread d'actualitzaci. L'utilitzarem per implementar
     * l'efecte de selecci, el d'animaci i el de gir (i la combinaci d'ells)
     */
    protected void onManagedUpdate(float time) {
    	/* SELECCI */
        if(this.getSelected()) {
            if(this._alphaMinor) {
            	this._alpha = (float)((double)this._alpha - 0.05D);
                if((double)this._alpha > 0.35D) {
                	this.setAlpha(this._alpha);
                } else {
                	this._alphaMinor = false;
                	this._alphaPlus = true;
                }
            }
            if(this._alphaPlus) {
            	this._alpha = (float)(0.05D + (double)this._alpha);
                if(this._alpha <= 1F) {
                	this.setAlpha(this._alpha);
                } else {
                	this._alphaMinor = true;
                	this._alphaPlus = false;
                }
            }
        }
        
        if(!this._flipAndAnimate) {
        	/* GIR (sense animaci) */
        	if(this._flip && !this._animate) {
                if(!this._activateFlipSound) {
                	//Quan girem la carta, fem sonar el so assignat al gir
                	this._activateFlipSound = true;
                	this._mCardFlipSound.play();
                }
                this._duration = 0.5F;
                this.UpdateFlip(time);
            }
        	/* GIR INVERS (sense animaci) */
            if(this.isInverseFlip() && !this._animate) {
                if(!this._activateFlipSound) {
                	//Quan girem la carta, fem sonar el so assignat al gir
                	this._activateFlipSound = true;
                	this._mCardFlipSound.play();
                }
                this._duration = 0.5F;
                this.UpdateFlipInverse(time);
            }
            /* ANIMACI */
            if(this._animate)
            	this.Transition(time);
            
            super.onManagedUpdate(time);
            return;   
            
        } else {
        	/* ANIMACI I GIR A L'HORA */
        	if(this._flip) {
                if(!this._activateFlipSound) {
                	this._activateFlipSound = true;
                	this._mCardFlipSound.play();
                }
                this._duration = 0.5F;
                this.UpdateFlip(time);
            }
            if(!this._flip && this._animate)
            	this.Transition(time);
        }
    }
    
    /**
     * Funci per saber si un punt pertany a una carta.
     * 
     * @param pX : X del punt
     * @param pY : Y del punt
     * @return True en cas afirmatiu
     */
    public boolean containsCoordinates(float pX, float pY) {
    	return ((this.mX <= pX) && (this.mX + SpriteCard.CARD_WIDTH) >= pX) &&
    		   ((this.mY <= pY) && (this.mY + SpriteCard.CARD_HEIGHT) >= pY);
    }

    /* Setters i getters */
    
    public void setAlpha(float alpha)
    {
    	this._selectedCard.setAlpha(alpha);
    	this._selectedCard.setColor(alpha, alpha, alpha);
    }

	public void setSelected(Boolean selected) {
		this._selected = selected;
	}

	public Boolean getSelected() {
		return this._selected;
	}

	public void setInverseFlip(boolean inverseFlip) {
		this._inverseFlip = inverseFlip;
	}

	public boolean isInverseFlip() {
		return this._inverseFlip;
	}

	public void setFlip(boolean flip) {
		this._flip = flip;
	}

	public boolean isFlip() {
		return this._flip;
	}
	
	public boolean isAnimate() {
		return this._animate;
	}

	public void setNewPosX(float newPosX) {
		this._newPosX = newPosX;
	}

	public float getNewPosX() {
		return _newPosX;
	}

	public void setNewPosY(float newPosY) {
		this._newPosY = newPosY;
	}

	public float getNewPosY() {
		return _newPosY;
	}
}
