package com.sweep15;

import java.util.Iterator;

import org.anddev.andengine.entity.IEntity;
import org.anddev.andengine.entity.text.Text;
import org.anddev.andengine.util.HorizontalAlign;

import com.sweep15.GameElements.Card;
import com.sweep15.GameElements.CardList;
import com.sweep15.GameElements.Deck;
import com.sweep15.GameElements.SpriteCard;

/**
 * Classe que implementa el controlador per al joc en mode local
 * 
 * @author Sergi Collado Figueras : PFC - Xarxes de computadors 2011-2012
 *
 */
public class GameLocalController {
	
	/* Enumerats */
	
	/**
	 * Indica si estem en el torn del jugador o de la CPU
	 */
	private static enum turnGame {
		HUMAN_PLAYER,
		CPU_PLAYER
	}

	/**
	 * Indica el moment de joc en que es troba la partida
	 */
	private static enum gameState {
		SHUFFLING,
		DEALING_CARDS,
		PLAYING,
		END_ROUND,
		GAME_OVER
	}
	
	/* Constants */

	public static float PLAYER_CAPTUREDCARDS_X = -20F;
	public static float PLAYER_CAPTUREDCARDS_Y = MainActivity.CAMERA_HEIGHT - SpriteCard.CARD_HEIGHT;
	public static float CPU_CAPTUREDCARDS_X = MainActivity.CAMERA_WIDTH - SpriteCard.CARD_WIDTH + 20F;
	public static float CPU_CAPTUREDCARDS_Y = 0F;
	
	public static float POINTS_TO_WIN = 21; //Juguem a 21 punts
	
	/* Propietats */
	  
	private Sweep15GameLocalActivity _gameLocalActivity;
	
	private Player _player; //Jugador hum
	private Player _cpu; //Jugador CPU
	private IACpu _iaCpu; //IA del jugador CPU
	
	private Deck _deckGame; //Baralla de cartes per jugar a l'escombra
	
	private turnGame _turnGame;
	private gameState _gameState;
	
	private CardList _cardsTable; //Cartes a la taula
	private CardList _playerLostCards; //Cartes perdudes
	
	private turnGame _lastPlayerWithValidGame; //Controla l'ltim jugador que ha fet jugada
	
	private boolean _showDeckCpu;
	private boolean _showDeckPlayer;
	private boolean _enableShuffleAnimation;
	private boolean _setScoreOnce;
	private boolean _continueGame;

	
	/* Constructor */
	     
	public GameLocalController(Sweep15GameLocalActivity gameLocalActivity) {
		/* Inicialitzacions */
		this._gameLocalActivity = gameLocalActivity;		
		
		this._setScoreOnce = false;
		/* Jugadors : Hum vs CPU */
		this._player = new Player("Player");
		this._cpu = new Player("Cpu");
		this._cpu.isDealer = true; //No ho sortejem, per donar cert avantatge al jugador hum
		/* Llistes de cartes d'utilitat */
		this._cardsTable = new CardList();		
		this._playerLostCards = new CardList();      
		this._deckGame = new Deck(this._gameLocalActivity.getPackage());
		this._deckGame.shuffle();
		this._iaCpu = new IACpu();
		this._gameState = gameState.SHUFFLING;
		this._turnGame = turnGame.HUMAN_PLAYER;
		this._enableShuffleAnimation = false;
		this._showDeckPlayer = false;
		this._showDeckCpu = false;
	    this._continueGame = false;
	} 

	/* Mtodes privats */
	
	/**
	 * Implementa un joc senzill de la CPU. Noms t en compte la millor jugada en el moment d'avaluar les
	 * cartes de la m i de la taula, sense tenir en compte qu ha sortit amb anterioritat.
	 */
	private void AIPlayEasy() {
		/* En primer lloc, seleccionem la millor jugada */
		CardList lCardList = new CardList(this._iaCpu.SelectedCardsByCPU(this._cpu.getHandPlayer(), this._cardsTable));
		/* Si noms tenim 1 carta en la millor selecci, vol dir que no tenim cap jugada vlida, per tant descartem 
		 * la carta de m que sigui menys favorable */
		if (lCardList.size() <= 1) {
			/* Descartem la carta menys favorable pel contrari */
			Card lCard = this._iaCpu.DiscardCards(this._cardsTable, this._cpu.getHandPlayer());
			this._cpu.getHandPlayer().remove(lCard);
		    lCard.getSprite().setFlip(true);
		    this._cardsTable.add(lCard);
		    this.SortCardsTable();
		} else {
			/* Entrem aqu si tenim alguna selecci de cartes per fer jugada */
			this._lastPlayerWithValidGame = turnGame.CPU_PLAYER;
			Card handCard = lCardList.get(0);
			CardList listCards = new CardList(lCardList);
			listCards.remove(0);
			/* Missatge d'escombra, si cal */
			if(listCards.size() == this._cardsTable.size())
				this.InitMsgGame(this._gameLocalActivity.getResources().getText(R.string.msgBROOM).toString());
			/* Actualitzem puntuaci */
			this._cpu.refreshScore(handCard, listCards, this._cardsTable.size());
			/* Moviment carta de m */
			handCard.getSprite().StartFlipAndAnimate(650F, 0F, 1.4F);
			handCard.getSprite().setInverseFlip(true);
			this._cpu.getCapturedCards().add(handCard);			
			this._cpu.getHandPlayer().remove(handCard);
			/* Moviment cartes de la taula */		      
			Iterator<Card> iterator = listCards.getIterator();
			while (iterator.hasNext()) {
				Card tableCard = iterator.next();
				this._cpu.getCapturedCards().add(tableCard);
				tableCard.getSprite().setInverseFlip(true);
				tableCard.getSprite().StartTransition(650F, 0F, 1.4F);
				this._cardsTable.remove(tableCard);
			}
		}	  
	}

	/**
	 * Reordena les cartes que hi ha a la taula, per evitar deixar forats buids
	 */
	private void SortCardsTable() {
		float originY = 10 + (Sweep15GameLocalActivity.CAMERA_HEIGHT - SpriteCard.CARD_HEIGHT / 2) / 2;
		float modifXeven = 0.0F; //Modificador per cartes parells
		float modifXodd = -1 * (5 + SpriteCard.CARD_WIDTH); //Modificador per cartes senars
		float modifY = 0F;
		for (int i = 0; i < this._cardsTable.size() ; i++) {
			/* Si l'ndex s parell, ubiquem a la dreta */
			if (i % 2 == 0) {
				this._cardsTable.get(i).getSprite().StartTransition((Sweep15GameLocalActivity.CAMERA_WIDTH / 2) + modifXeven, originY + modifY, 0.4F);
				modifXeven += 5 + SpriteCard.CARD_WIDTH;
			/* Si l'ndex s imparell ubiquem a l'esquerra */
			} else {
				this._cardsTable.get(i).getSprite().StartTransition((Sweep15GameLocalActivity.CAMERA_WIDTH / 2) + modifXodd, originY + modifY, 0.4F);
				modifXodd -= 5 + SpriteCard.CARD_WIDTH;
			}
			/* Si passem de 6 cartes, tornem a organitzar-les comenant pel mig, per en la fila superior */
			if (i == 6) {
				modifXeven = 0.0F;
				modifXodd = -1 * (5 + SpriteCard.CARD_WIDTH);
				modifY = -1 * (5 + SpriteCard.CARD_HEIGHT);
			}			
		}
	}
	
	/**
	 * Mostra un missatge de joc en gran.
	 * 
	 * @param pMessage : Missatge a mostrar
	 */
	private void InitMsgGame(String pMessage) {
		this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.msgGame);
		this._gameLocalActivity.msgGame.InitMsgGame(pMessage, (MainActivity.CAMERA_WIDTH / 2) - 14 * pMessage.length() / 2, 160, this._gameLocalActivity.mFrankFont, 0.6F);
		this._gameLocalActivity.mMainScene.getLastChild().attachChild(this._gameLocalActivity.msgGame);
	} 
	
	/**
	 * Retorna true si hi ha alguna animaci en marxa en les cartes que son susceptibles
	 * d'estar animades en aquest moment
	 * 
	 * @return true si hi ha alguna
	 */
	private boolean CheckAnimations() {
		/* Examinem totes les cartes ams sprites per si hi ha animacions en marxa */
		boolean runsAnyanimation = false;
		runsAnyanimation |= this._cardsTable.isRunningAnimation();
		if (runsAnyanimation) {return true;}
		runsAnyanimation |= this._player.getCapturedCards().isRunningAnimation();
		if (runsAnyanimation) {return true;}
		runsAnyanimation |= this._cpu.getCapturedCards().isRunningAnimation();
		if (runsAnyanimation) {return true;}
		runsAnyanimation |= this._player.isAnimating();
		if (runsAnyanimation) {return true;}
		runsAnyanimation |= this._cpu.isAnimating();
		if (runsAnyanimation) {return true;}
		return false;	
	}
	
	/**
	 * Comproba si les cartes de la taula formen una escombra. En cas afirmatiu, assigna les cartes i els
	 * punts al jugador que feia de repartidor.
	 * 
	 * @return true si ha trobat una escombra
	 */
	private Boolean CheckEscobaHandDealer() {
		boolean escobaFound = false;
		//TODO Actualment busquem noms una escombra de ma, caldria contemplar la possibilitat de tenir-ne 2.
		if (this._cardsTable.getRankSum() == 15) {
			/* En funci del que reparteix, caldr enviar la escombra de m cap a un lloc o un altre */
			Iterator<Card> iter = this._cardsTable.getIterator();
			/* Transicions de les cartes */
			while (iter.hasNext()) {
				Card lCard = iter.next();
				lCard.getSprite().setInverseFlip(true);
				if (this._cpu.isDealer) {
					//Si repartia la CPU
					this._cpu.getCapturedCards().add(lCard);
					lCard.getSprite().StartTransition(CPU_CAPTUREDCARDS_X, CPU_CAPTUREDCARDS_Y, 0.3F);
				} else {
					//Si repartia el jugador
					this._player.getCapturedCards().add(lCard);
					lCard.getSprite().StartTransition(PLAYER_CAPTUREDCARDS_X, PLAYER_CAPTUREDCARDS_Y, 0.3F);
				}
			}
			//Refresquem puntuacions
			if (this._cpu.isDealer) {	
				this._cpu.refreshScoreEscobaHandDealer(this._cardsTable); //Actualitzem les puntuacions de la CPU
			} else {
				this._player.refreshScoreEscobaHandDealer(this._cardsTable); //Actualitzem les puntuacions del jugador
			}
			escobaFound = true;
			this._cardsTable.clear(); //Eliminem les cartes de la llista de cartes de taula
		}
		return escobaFound;
	}
	
	/**
	 * Reparteix les cartes als jugadors
	 */
	private void DealCards() {
		/* Si son les ltimes, mostrem un missatge */
		if (this._deckGame.deckSize() == 6) {
			InitMsgGame(this._gameLocalActivity.getResources().getText(R.string.msgLAST_CARDS).toString());
			//i eliminem el sprite de la baralla
			this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.deck);
		}
		this.DealToPlayer();
		this.DealToCpu();
	}
	
	/**
	 * Reparteix les cartes a la CPU
	 */
	private void DealToCpu() {
		float originX = (MainActivity.CAMERA_WIDTH - 3 * SpriteCard.CARD_WIDTH - 40) / 2F;
		float originY = -40F;
		//Ubiquem les 3 cartes
	    for (int i = 0; i < 3; i++) {
	    	Card card = this._deckGame.extractNextCard(); //Extraiem la segent carta de la baralla
		    card.getSprite().setNewPosX(originX + i * (SpriteCard.CARD_WIDTH + 20));
		    card.getSprite().setNewPosY(originY);
		    this._cpu.getHandPlayer().add(card); //Afegim la carta a les cartes de la m de la cpu
		    this._cpu.getHandPlayer().get(i).getSprite().StartTransition(card.getSprite().getNewPosX(), card.getSprite().getNewPosY(), 0.8F); //Moviment (animaci)
			this._gameLocalActivity.mMainScene.getLastChild().attachChild((IEntity)this._cpu.getHandPlayer().get(i).getSprite()); //Inserim el sprite a l'escena
			//Nota: segent lnia obsoleta, al final es resol directament en l'event OnTouchArea de l'escena. Es deixa constncia per
			//si cal tornar-ho a canviar ms endavant.
			//this._gameLocalActivity.mMainScene.registerTouchArea((Scene.ITouchArea)this._player.handPlayer.get(i).getSprite());
	    }    
	}

	/**
	 * Reparteix les cartes al jugador hum
	 */
	private void DealToPlayer() {
		float originX = (MainActivity.CAMERA_WIDTH - 3 * SpriteCard.CARD_WIDTH - 40) / 2F;
	    float originY = MainActivity.CAMERA_HEIGHT - SpriteCard.CARD_HEIGHT;
	    //Ubiquem les 3 cartes
	    for (int i = 0; i < 3; i++) {
	    	Card card = this._deckGame.extractNextCard(); //Extraiem la segent carta de la baralla
		    card.getSprite().setNewPosX(originX + i * (SpriteCard.CARD_WIDTH + 20));
		    card.getSprite().setNewPosY(originY);		    
		    this._player.getHandPlayer().add(card); //Afegim la carta a les cartes de la m del jugador hum
		    this._player.getHandPlayer().get(i).getSprite().StartTransition(card.getSprite().getNewPosX(), card.getSprite().getNewPosY(), 0.8F);
		    this._player.getHandPlayer().get(i).getSprite().setFlip(true);	    
			this._gameLocalActivity.mMainScene.getLastChild().attachChild((IEntity)this._player.getHandPlayer().get(i).getSprite());
			//Nota: segent lnia obsoleta, al final es resol directament en l'event OnTouchArea de l'escena. Es deixa constncia per
			//si cal tornar-ho a canviar ms endavant.
			//this._gameLocalActivity.mMainScene.registerTouchArea((Scene.ITouchArea)this_.player.getHandPlayer().get(i).getSprite());
	    }
	  }
	
	/**
	 * Recull les ltimes cartes de la taula, i les assigna a l'ltim jugador que va recollir
	 * una bassa.
	 */
	private void CleanLastCards() {
		/* L'ltim jugador a fer jugada es queda les ltimes cartes */	  
	    if (this._lastPlayerWithValidGame == turnGame.HUMAN_PLAYER)
	    	this._player.refreshScoreLastCards(this._cardsTable);
	    else 
	    	this._cpu.refreshScoreLastCards(this._cardsTable);
	    
	    /* Eliminem les cartes de la m (i els sprites) */
	    Iterator<Card> iterator = this._cardsTable.getIterator();
	    while (iterator.hasNext()) {
	    	Card card = (Card)iterator.next();
	        //this._gameLocalActivity.mMainScene.unregisterTouchArea(card.getSprite());
	        this._gameLocalActivity.mMainScene.getLastChild().detachChild(card.getSprite());
	    }
	    this._cardsTable.clear();
	}	
	
	/**
	 * Desseleccionem totes les cartes de la taula
	 */
	private void DeselectCardsTable() {
		/* Treiem la marca de selecci de les cartes de taula */
		this._cardsTable.unSelectAll();
	}

	/**
	 * Elimina i captura tant la carta de m seleccionada del jugador, com les cartes de la taula passades com a parmetre
	 * 
	 * @param pCardsToErase : llista de cartes de la taula a capturar pel jugador hum
	 */
	private void ErasePlayerCards(CardList pCardsToErase) {
		/* Eliminem la carta seleccionada de la ma del jugador */
		//this._gameLocalActivity.mMainScene.unregisterTouchArea((Scene.ITouchArea)this._player.getHandPlayer().get(this._player.getCardSelected()).getSprite());
		this._player.getHandPlayer().get(this._player.getCardSelected()).getSprite().setSelected(false);		
		this._player.getHandPlayer().get(this._player.getCardSelected()).getSprite().setInverseFlip(true);
		this._player.getHandPlayer().get(this._player.getCardSelected()).getSprite().StartTransition(PLAYER_CAPTUREDCARDS_X, PLAYER_CAPTUREDCARDS_Y, 0.3F);
		//Passem la carta seleccionada a les cartes capturades del jugador
		this._player.getCapturedCards().add(this._player.getHandPlayer().remove(this._player.getCardSelected()));
		this._player.setCardSelected(-1); //Ara ja no tenim cap carta seleccionada
		    
		/* Eliminem les cartes seleccionades de la taula */
		while (pCardsToErase.size() > 0) {
			//int iter = this._cardsTable.indexOf(pCardsToErase.get(0));
			//this._gameLocalActivity.mMainScene.unregisterTouchArea((Scene.ITouchArea)this._cardsTable.get(iter).getSprite());
			pCardsToErase.get(0).getSprite().setSelected(false);        
			pCardsToErase.get(0).getSprite().setInverseFlip(true);
			pCardsToErase.get(0).getSprite().StartTransition(PLAYER_CAPTUREDCARDS_X, PLAYER_CAPTUREDCARDS_Y, 0.3F);
			this._player.getCapturedCards().add(pCardsToErase.get(0));
			this._cardsTable.remove(pCardsToErase.get(0));
			pCardsToErase.remove(0);
		}
	}

	/**
	 * Comproba si una jugada (carta de m + llista de cartes de la taula) s vlida, s a dir, suma 15 punts.
	 * 
	 * @param pHandCard : carta de m
	 * @param pSelectedCards : llista de cartes de taula
	 * @return true en cas que facin una jugada legal
	 */
	private Boolean IsValid(Card pHandCard, CardList pSelectedCards) {
		int valueCount = pHandCard.getRank().getRankNumber(); //Rang de la carta de m
		/* Recorrem les cartes seleccionades i acumulem els rangs */
		Iterator<Card> iterator = pSelectedCards.getIterator();	  	 
		while (iterator.hasNext() && valueCount <= 15) {
			valueCount += iterator.next().getRank().getRankNumber();
		}
		/* La selecci ser vlida quan la suma de rangs sigui exactament 15 */
		return (valueCount == 15);    
	}

	/**
	 * Netejem sprites innecessaris de la llista de cartes capturades de la CPU (per optimitzar recursos grfics)
	 * Nota: Un cop s'han computeritzat els punts no en fem rs d'aquestes cartes capturades, per si les deixem
	 * 		 apilades consumeixen fora recursos.
	 */
	private void OnlyShowOneCardCpuDeck() {
		/* Si la CPU t alguna bassa, per encara no mostra el sprite amb carta nica, el mostrem */
		if(this._cpu.getCapturedCards().size() > 1 && !this._showDeckCpu) {
			this._showDeckCpu = true;
			this._gameLocalActivity.spriteDeckCpu.setPosition(CPU_CAPTUREDCARDS_X, CPU_CAPTUREDCARDS_Y);
			this._gameLocalActivity.mMainScene.getLastChild().attachChild(this._gameLocalActivity.spriteDeckCpu);
		}
		/* Eliminem les cartes de la m (i els sprites) */
		Iterator<Card> iterator = this._cpu.getCapturedCards().getIterator();
		while (iterator.hasNext()) {
			Card card = (Card)iterator.next();
			//this._gameLocalActivity.mMainScene.unregisterTouchArea(card.getSprite());
			this._gameLocalActivity.mMainScene.getLastChild().detachChild(card.getSprite());
		}
		this._cpu.getCapturedCards().clear();
	}

	/**
	 * Netejem sprites innecessaris de la llista de cartes capturades del jugador hum (per optimitzar recursos grfics)
	 * Nota: Un cop s'han computeritzat els punts no en fem rs d'aquestes cartes capturades, per si les deixem
	 * 		 apilades consumeixen fora recursos.
	 */
	private void OnlyShowOnecardPlayerDeck() {
		/* Si el jugador hum t alguna bassa, per encara no mostra el sprite amb carta nica, el mostrem */
		if(this._player.getCapturedCards().size() > 1 && !this._showDeckPlayer) {
			this._showDeckPlayer = true;
			this._gameLocalActivity.spriteDeckPlayer.setPosition(PLAYER_CAPTUREDCARDS_X, PLAYER_CAPTUREDCARDS_Y);
			this._gameLocalActivity.mMainScene.getLastChild().attachChild(this._gameLocalActivity.spriteDeckPlayer);
		}
		/* Eliminem les cartes de la m (i els sprites amb les seves rees de contacte) */
		Iterator<Card> iterator = this._player.getCapturedCards().getIterator();
		while (iterator.hasNext()) {
			Card card = (Card)iterator.next();
			//this._gameLocalActivity.mMainScene.unregisterTouchArea(card.getSprite());
			this._gameLocalActivity.mMainScene.getLastChild().detachChild(card.getSprite());
		}
		this._player.getCapturedCards().clear();
	}

	/**
	 * Retorna una llista amb les cartes seleccionades del conjunt de cartes passat per parmetre
	 * @param pCardsToCheck : cartes de les que es vol trobar les que estan seleccionades
	 * @return llista de cartes seleccionades
	 */
	private CardList SelectedCards(CardList pCardsToCheck) {
		CardList selection = new CardList();
		Iterator<Card> iterator = pCardsToCheck.getIterator();
		while (iterator.hasNext()) {
			Card lCard = iterator.next();
			if (lCard.getSprite().getSelected())
				selection.add(lCard);
		}	
		return selection;
	}
	
	/**
	 * Eliminem la carta de m del jugador i la afegim a les cartes de la taula
	 */
	private void ThrowCard() {
		Card lCard = this._player.getHandPlayer().get(this._player.getCardSelected());
		//this._gameLocalActivity.mMainScene.unregisterTouchArea((Scene.ITouchArea)this._player.getHandPlayer().get(this._player.getCardSelected()).getSprite());
		/* Eliminem la carta seleccionada pel jugador de les seves cartes de ma */
		this._player.getHandPlayer().remove(this._player.getCardSelected());
		this._player.setCardSelected(-1); //Actualitzem la selecci
		lCard.getSprite().setSelected(false);		
		/* Afegim la carta a les cartes de la taula */
		this._cardsTable.add(lCard);
	}
	
	/**
	 * Repartiment de cartes inicial de les cartes de taula
	 */
	private void initDeal()	{		
		float originX = (MainActivity.CAMERA_WIDTH - 4F * (SpriteCard.CARD_WIDTH + 10)) / 2F;
	    float originY = 10F + (MainActivity.CAMERA_HEIGHT - SpriteCard.CARD_HEIGHT / 2F) / 2F; //desplaat a la part inferior
	    for (int i = 0; i < 4; i++) {
	    	Card card = this._deckGame.extractNextCard(); //Extraiem la segent carta de la baralla
	    	card.getSprite().setNewPosX(originX + i * (SpriteCard.CARD_WIDTH + 10));
	    	card.getSprite().setNewPosY(originY);
	    	this._cardsTable.add(card); //Afegim la carta a la taula
	    	this._cardsTable.get(i).getSprite().StartTransition(card.getSprite().getNewPosX(), card.getSprite().getNewPosY(), 0.8F); //Moviment (animaci)
	    	this._cardsTable.get(i).getSprite().setFlip(true); //Gir	    	
	    	this._gameLocalActivity.mMainScene.getLastChild().attachChild((IEntity)this._cardsTable.get(i).getSprite());
	    	//this._gameLocalActivity.mMainScene.registerTouchArea((Scene.ITouchArea)this._cardsTable.get(i).getSprite());
	    }
	  }

	
	/**
	 * Reteja de tots els elements visuals referents a mostrar la puntuaci al final de la ronda
	 */
	private void QuitGameScores() {
		/* Netejem tots els elements */
		this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.tTouchContinue);
	    Iterator<Text> iterScore = this._gameLocalActivity.tScore.iterator();
	    while (iterScore.hasNext()) {
	    	Text scoreText = iterScore.next();
	    	this._gameLocalActivity.mMainScene.getLastChild().detachChild(scoreText);
	    }
	    this._gameLocalActivity.tScore.clear();	    
	}
	
	/**
	 * Contabilitzem els punts obtinguts a la ronda, i actualitzem els totals de cada jugador
	 */
	private void CountPoints() {
		//Contem els punts obtinguts en la ronda: primer atorguem les escombres
		int pointsPlayer = this._player.getScore().getBroomCount();
		int pointsCPU = this._cpu.getScore().getBroomCount();
		
		//Comparem la resta de conceptes
		if (this._player.getScore().getCardsCount() > this._cpu.getScore().getCardsCount()) {
			pointsPlayer++;
		} else if (this._player.getScore().getCardsCount() < this._cpu.getScore().getCardsCount()) {
			pointsCPU++;
		}
		if (this._player.getScore().getGoldCount() > this._cpu.getScore().getGoldCount()) {
			pointsPlayer++;
		} else if (this._player.getScore().getGoldCount() < this._cpu.getScore().getGoldCount()) {
			pointsCPU++;
		}
		if (this._player.getScore().getSevenCount() > this._cpu.getScore().getSevenCount()) {
			pointsPlayer++;
		} else if (this._player.getScore().getSevenCount() < this._cpu.getScore().getSevenCount()) {
			pointsCPU++;
		}
		//Nota: tot i que noms hi ha 1 7 d'oros, ho fem igual
		if (this._player.getScore().getSevenGold() > this._cpu.getScore().getSevenGold()) {
			pointsPlayer++;
		} else if (this._player.getScore().getSevenGold() < this._cpu.getScore().getSevenGold()) {
			pointsCPU++;
		}
		//Sumem els punts
		this._player.getScore().addTotalScore(pointsPlayer);
		this._cpu.getScore().addTotalScore(pointsCPU);		
	}
	
	/**
	 * Mostrem les puntuacions en un panell
	 */
	private void ShowGameScores() {	
		this._gameLocalActivity.mAmbience.play();
		float oXply = 405F;
		float oXcpu = 570F;
		float oY = 165F;
		float ySepar = 40F;
		float leftCaptions = 191F;
		
	    Text captionPlayer = new Text(oXply - 49F, oY - 1 * ySepar, this._gameLocalActivity.mFont, 
	    						 this._gameLocalActivity.getResources().getText(R.string.scoPLAYER).toString(), HorizontalAlign.LEFT);
	    captionPlayer.setColor(1F, 0F, 0F);
	    
	    Text captionCPU = new Text(oXcpu - 21F, oY - 1 * ySepar, this._gameLocalActivity.mFont, 
	    						 this._gameLocalActivity.getResources().getText(R.string.scoCPU).toString(), HorizontalAlign.LEFT);
	    captionCPU.setColor(1F, 0F, 0F);
	    
	    Text captionPoints = new Text(oXply - leftCaptions, oY, this._gameLocalActivity.mFont, 
	    						 this._gameLocalActivity.getResources().getText(R.string.scoPOINTS).toString(), HorizontalAlign.LEFT);	    	  
	    captionPoints.setColor(0F, 0.7F, 0F);
	    
	    Text captionBrooms = new Text(oXply - leftCaptions, oY + 1 * ySepar, this._gameLocalActivity.mFont, 
	    						 this._gameLocalActivity.getResources().getText(R.string.scoBROOMS).toString(), HorizontalAlign.LEFT);	    	  
	    captionBrooms.setColor(0F, 0F, 1F);
	    
	    Text captionSevens = new Text(oXply - leftCaptions, oY + 2 * ySepar, this._gameLocalActivity.mFont, 
				 				 this._gameLocalActivity.getResources().getText(R.string.scoSEVENS).toString(), HorizontalAlign.LEFT);	    	  
	    captionSevens.setColor(0F, 0F, 1F);

	    Text captionGoldSeven = new Text(oXply - leftCaptions, oY + 3 * ySepar, this._gameLocalActivity.mFont, 
	    						 this._gameLocalActivity.getResources().getText(R.string.scoGOLD_SEVEN).toString(), HorizontalAlign.LEFT);	    	  
	    captionGoldSeven.setColor(0F, 0F, 1F);

	    Text captionGold = new Text(oXply - leftCaptions, oY + 4 * ySepar, this._gameLocalActivity.mFont, 
	    						 this._gameLocalActivity.getResources().getText(R.string.scoGOLDS).toString(), HorizontalAlign.LEFT);	    	  
	    captionGold.setColor(0F, 0F, 1F);

	    Text captionCards = new Text(oXply - leftCaptions, oY + 5 * ySepar, this._gameLocalActivity.mFont, 
	    						 this._gameLocalActivity.getResources().getText(R.string.scoCARDS).toString(), HorizontalAlign.LEFT);	    	  
	    captionCards.setColor(0F, 0F, 1F);
	    
	    Text playerPoints = new Text(oXply + 49F, oY - 1 * ySepar, this._gameLocalActivity.mFont, " " + this._player.getScore().getTotalScore(), HorizontalAlign.LEFT);
	    playerPoints.setColor(0F, 0F, 0F);
	    
	    Text cpuPoints = new Text(oXcpu + 31F, oY - 1 * ySepar, this._gameLocalActivity.mFont, " " + this._cpu.getScore().getTotalScore(), HorizontalAlign.LEFT);
	    cpuPoints.setColor(0F, 0F, 0F);
	    
	    Text playerPointsBis = new Text(oXply, oY, this._gameLocalActivity.mFont, String.valueOf(this._player.getScore().getTotalScore()), HorizontalAlign.LEFT);
	    playerPointsBis.setColor(0F, 0.7F, 0F);
	    
	    Text cpuPointsBis = new Text(oXcpu, oY, this._gameLocalActivity.mFont, String.valueOf(this._cpu.getScore().getTotalScore()), HorizontalAlign.LEFT);
	    cpuPointsBis.setColor(0F, 0.7F, 0F);
	    
	    Text playerBrooms = new Text(oXply, oY + 1 * ySepar, this._gameLocalActivity.mFont, String.valueOf(this._player.getScore().getBroomCount()), HorizontalAlign.LEFT);
	    playerBrooms.setColor(0F, 0F, 1F);
	    
	    Text cpuBrooms = new Text(oXcpu, oY + 1 * ySepar, this._gameLocalActivity.mFont, String.valueOf(this._cpu.getScore().getBroomCount()), HorizontalAlign.LEFT);
	    cpuBrooms.setColor(0F, 0F, 1F);
	    
	    Text playerSevens = new Text(oXply, oY + 2 * ySepar, this._gameLocalActivity.mFont, String.valueOf(this._player.getScore().getSevenCount()), HorizontalAlign.LEFT);
	    playerSevens.setColor(0F, 0F, 1F);
	    
	    Text cpuSevens = new Text(oXcpu, oY + 2 * ySepar, this._gameLocalActivity.mFont, String.valueOf(this._cpu.getScore().getSevenCount()), HorizontalAlign.LEFT);
	    cpuSevens.setColor(0F, 0F, 1F);
	    
	    Text playerGoldSeven = new Text(oXply, oY + 3 * ySepar, this._gameLocalActivity.mFont, String.valueOf(this._player.getScore().getSevenGold()), HorizontalAlign.LEFT);
	    playerGoldSeven.setColor(0F, 0F, 1F);
	    
	    Text cpuGoldSeven = new Text(oXcpu, oY + 3 * ySepar, this._gameLocalActivity.mFont, String.valueOf(this._cpu.getScore().getSevenGold()), HorizontalAlign.LEFT);
	    cpuGoldSeven.setColor(0F, 0F, 1F);
	    	   	   
	    Text playerGold = new Text(oXply, oY + 4 * ySepar, this._gameLocalActivity.mFont, String.valueOf(this._player.getScore().getGoldCount()), HorizontalAlign.LEFT);
	    playerGold.setColor(0F, 0F, 1F);
	    
	    Text cpuGold = new Text(oXcpu, oY + 4 * ySepar, this._gameLocalActivity.mFont, String.valueOf(this._cpu.getScore().getGoldCount()), HorizontalAlign.LEFT);
	    cpuGold.setColor(0F, 0F, 1F);
	    
	    Text playerCards = new Text(oXply, oY + 5 * ySepar, this._gameLocalActivity.mFont, String.valueOf(this._player.getScore().getCardsCount()), HorizontalAlign.LEFT);
	    playerCards.setColor(0F, 0F, 1F);
	    
	    Text cpuCards = new Text(oXcpu, oY + 5 * ySepar, this._gameLocalActivity.mFont, String.valueOf(this._cpu.getScore().getCardsCount()), HorizontalAlign.LEFT);
	    cpuCards.setColor(0F, 0F, 1F);
	    
	    this._gameLocalActivity.tScore.add(captionPoints);
	    this._gameLocalActivity.tScore.add(captionPlayer);
	    this._gameLocalActivity.tScore.add(captionCPU);
	    this._gameLocalActivity.tScore.add(captionBrooms);
	    this._gameLocalActivity.tScore.add(captionSevens);
	    this._gameLocalActivity.tScore.add(captionGoldSeven);
	    this._gameLocalActivity.tScore.add(captionGold);
	    this._gameLocalActivity.tScore.add(captionCards);
	    this._gameLocalActivity.tScore.add(playerPoints);
	    this._gameLocalActivity.tScore.add(cpuPoints);	    
	    this._gameLocalActivity.tScore.add(playerPointsBis);
	    this._gameLocalActivity.tScore.add(cpuPointsBis);
	    this._gameLocalActivity.tScore.add(playerBrooms);
	    this._gameLocalActivity.tScore.add(cpuBrooms);
	    this._gameLocalActivity.tScore.add(playerSevens);
	    this._gameLocalActivity.tScore.add(cpuSevens);
	    this._gameLocalActivity.tScore.add(playerGoldSeven);
	    this._gameLocalActivity.tScore.add(cpuGoldSeven);
	    this._gameLocalActivity.tScore.add(playerGold);
	    this._gameLocalActivity.tScore.add(cpuGold);
	    this._gameLocalActivity.tScore.add(playerCards);
	    this._gameLocalActivity.tScore.add(cpuCards);
	    
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(captionPlayer);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(captionCPU);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(captionPoints);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(captionBrooms);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(captionSevens);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(captionGoldSeven);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(captionGold);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(captionCards);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(playerPoints);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(cpuPoints);	    
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(playerPointsBis);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(cpuPointsBis);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(playerBrooms);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(cpuBrooms);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(playerSevens);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(cpuSevens);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(playerGoldSeven);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(cpuGoldSeven);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(playerGold);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(cpuGold);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(playerCards);
	    this._gameLocalActivity.mMainScene.getLastChild().attachChild(cpuCards);
	    
	    if ((this._gameState == gameState.END_ROUND) || (this._gameState == gameState.GAME_OVER))
	    	this._gameLocalActivity.mMainScene.getLastChild().attachChild(this._gameLocalActivity.tTouchContinue);
	}
	
	/**
	 * Resetejem els elements necessaris per comenar una nova ronda del joc
	 */
	private void initRound() {
		this._deckGame = new Deck(this._gameLocalActivity.getPackage());
		//this._deckGame.shuffle(); //no cal, ja es fa al constructor
	    this._enableShuffleAnimation = false;
	    this._showDeckCpu = false;
	    this._showDeckPlayer = false;  
	    this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.spriteDeckPlayer);
	    this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.spriteDeckCpu);	
	    
	    this._player.getScore().reset();
	    this._cpu.getScore().reset();
	    
	    this._cardsTable.clear();
	    this._player.getHandPlayer().clear();
	    this._player.getCapturedCards().clear();
	    this._cpu.getHandPlayer().clear();
	    this._cpu.getCapturedCards().clear();	
	    
	    this._gameLocalActivity.deckPosY = (MainActivity.CAMERA_HEIGHT - this._gameLocalActivity.mDeckTextureRegion.getHeight()) / 2;
	}
	
	/**
	 * Configurem i mostrem el missatge final de la partida, en funci de si s'ha guanyat o si s'ha perdut
	 */
	private void SetMsgGameOver() {
		Text lText1 = new Text(270.0F - 49.0F, 150.0F - 50.0F, this._gameLocalActivity.mFont, 
							this._gameLocalActivity.getResources().getText(R.string.scoPLAYER).toString(), HorizontalAlign.LEFT);
		lText1.setColor(0.0F, 0.0F, 0.0F);
		
		Text lText2 = new Text(480.0F - 21.0F, 150.0F - 50.0F, this._gameLocalActivity.mFont, 
							this._gameLocalActivity.getResources().getText(R.string.scoCPU).toString(), HorizontalAlign.LEFT);
		lText2.setColor(0.0F, 0.0F, 0.0F);

		this._gameLocalActivity.tScore.add(lText1);
		this._gameLocalActivity.tScore.add(lText2);
		this._gameLocalActivity.mMainScene.getLastChild().attachChild(lText1);
		this._gameLocalActivity.mMainScene.getLastChild().attachChild(lText2);
		
		if (this._player.getScore().getTotalScore() > this._cpu.getScore().getTotalScore()) {
			this._gameLocalActivity.tMsgGameOver = new Text(290F, 400F, this._gameLocalActivity.mFont, 
							this._gameLocalActivity.getResources().getText(R.string.gmoYOU_WIN).toString(), HorizontalAlign.LEFT);
			this._gameLocalActivity.tMsgGameOver.setColor(0.0F, 0.7F, 0.0F);
			this._gameLocalActivity.trophy.setPosition(270F - this._gameLocalActivity.trophy.getWidth() / 2F, 150F);
			this._gameLocalActivity.mCheer.play();
			//this._gameLocalActivity.tMsgGameOver.setPosition(290F, 380F);
			
		} else if (this._player.getScore().getTotalScore() < this._cpu.getScore().getTotalScore()) {
			this._gameLocalActivity.tMsgGameOver = new Text(310F, 400F, this._gameLocalActivity.mFont, 
							this._gameLocalActivity.getResources().getText(R.string.gmoYOU_LOOSE).toString(), HorizontalAlign.LEFT);
			this._gameLocalActivity.tMsgGameOver.setColor(1.0F, 0.0F, 0.0F);
			this._gameLocalActivity.trophy.setPosition(480F - this._gameLocalActivity.trophy.getWidth() / 2F, 150F);
			this._gameLocalActivity.mBoo.play();
		} else {
			this._gameLocalActivity.tMsgGameOver = new Text(310F, 400F, this._gameLocalActivity.mFont, 
							this._gameLocalActivity.getResources().getText(R.string.gmoDRAW).toString(), HorizontalAlign.LEFT);
			this._gameLocalActivity.tMsgGameOver.setColor(0.0F, 0.0F, 0.0F);
			this._gameLocalActivity.trophy.setPosition(380F - this._gameLocalActivity.trophy.getWidth() / 2F, 150F);
			this._gameLocalActivity.mStream.play();
		}
		this._gameLocalActivity.mMainScene.getLastChild().attachChild(this._gameLocalActivity.tMsgGameOver);
		this._gameLocalActivity.mMainScene.getLastChild().attachChild(this._gameLocalActivity.trophy);	
	}
	
	/**
	 * Obt les cartes d'una jugada que puguin formar les cartes perdudes (quan el jugador tira una carta).
	 * til per saber si hi ha una bassa de taula.
	 * 
	 * @param pHandCard
	 * @param pTableCards
	 */
	private void CheckPlayerThrowCard(Card pHandCard, CardList pTableCards) {
		this._playerLostCards = this._iaCpu.CheckRejectedCardPlayer(pHandCard, pTableCards);
	}
	
	
	/**
	 * Funci que chequeja la posici premuda, i selecciona/deselecciona oportunament la carta corresponent 
	 * de taula o de la m del jugador hum que comprengui aquesta posici.
	 * 
	 * @param mTouchX : coordinada X de la posici premuda
	 * @param mTouchY : coordinada Y de la posici premuda
	 */
	protected void checkTouchPointOnCards(float mTouchX, float mTouchY) {
		/* Busquem la carta de la m del jugador que ha estat premuda */
		boolean cardFound = false;
		for (int i = 0; i < this._player.getHandPlayer().size(); i++) {
			/* Si la carta que es toca no estava seleccionada, la seleccionem */
			if(this._player.getHandPlayer().get(i).getSprite().containsCoordinates(mTouchX, mTouchY) && (i != this._player.getCardSelected())) {
				this._player.getHandPlayer().get(i).getSprite().setSelected(true);
				/* Desselecionem la carta que tenia seleccionada el jugador, i actualitzem */
				if(this._player.getCardSelected() != -1)
					this._player.getHandPlayer().get(this._player.getCardSelected()).getSprite().setSelected(false);
				this._player.setCardSelected(i);	       
				cardFound = true;
			} else if(this._player.getHandPlayer().get(i).getSprite().containsCoordinates(mTouchX, mTouchY) && i == this._player.getCardSelected()) {
			/* Si toquen una carta seleccionada, la deseleccionem */
				this._player.setCardSelected(-1);
				this._player.getHandPlayer().get(i).getSprite().setSelected(false);
      		  	cardFound = true;
			}  
		}
		/* Busquem la carta de la taula que ha estat premuda (si no s'ha pres cap de la m del jugador) */
		if (!cardFound) {
			for (int i = 0; i < this._cardsTable.size(); i++) {
				/* Si la carta que es toca no estava seleccionada, la seleccionem */
				if(this._cardsTable.get(i).getSprite().containsCoordinates(mTouchX, mTouchY)) 
					this._cardsTable.get(i).getSprite().setSelected(!this._cardsTable.get(i).getSprite().getSelected());		                
			}
		}		
	}
	
	
	/**
	 * Lgica PRINCIPAL del joc de l'escombra
	 */
	protected void gameStateProcess() {
		
		//Nota: la lgica de joc sempre actuar quan no hi hagi cap animaci de cap carta, ja que
		//      aquest mtode es cridar en el OnUpdate del thread principal del AndEngine.
		
		switch (this._gameState) {
		
		/* Estat : BARREJAR CARTES */
		case SHUFFLING:
			if(!this._enableShuffleAnimation) {
				/* Si no estem barrejant (visualment) habilitem tots els sprites i reproduim els sons */
                this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.deck);
                this._setScoreOnce = false;
                this._gameLocalActivity.mShuffleSound.play();
                this._enableShuffleAnimation = true;
                this._gameLocalActivity.ShuffleDeck.animate(50L, 1);
                this._gameLocalActivity.mMainScene.getLastChild().attachChild(this._gameLocalActivity.ShuffleDeck);
                //Ubiquem la marca de repartidor en la posici de la banda del jugador que reparteix
                if(this._cpu.isDealer) {
                    float posY = (MainActivity.CAMERA_HEIGHT / 2) - this._gameLocalActivity.mDeckTextureRegion.getHeight();
                    this._gameLocalActivity.dealer.setPosition(this._gameLocalActivity.dealerPosX, posY);
                    this._cpu.isDealer = false;
                    this._turnGame = turnGame.HUMAN_PLAYER;
                } else {
                    float posY = (MainActivity.CAMERA_HEIGHT / 2) + (15 + this._gameLocalActivity.mDeckTextureRegion.getHeight() / 2);
                    this._gameLocalActivity.dealer.setPosition(this._gameLocalActivity.dealerPosX, posY);
                    this._cpu.isDealer = true;
                    this._turnGame = turnGame.CPU_PLAYER;
                }
            }
			/* Quan s'acabi l'animaci de barrejar, deixem la baralla normal, i mostrem missatge */
            if(!this._gameLocalActivity.ShuffleDeck.isAnimationRunning()) {
            	this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.ShuffleDeck);
            	this._gameLocalActivity.mMainScene.getLastChild().attachChild(this._gameLocalActivity.deck);
                this.InitMsgGame(this._gameLocalActivity.getResources().getText(R.string.msgGAME_BEGINS).toString());
                //Passem al segent estat
                this._gameState = gameState.DEALING_CARDS;
            }
			break;
			
		/* Estat : REPARTIR */
		case DEALING_CARDS:
			if(!this.CheckAnimations() && this._player.getHandPlayer().isEmpty())
				//Si el jugador no t cartes, li donem
				this.DealToPlayer();
            if(!this.CheckAnimations() && this._cpu.getHandPlayer().isEmpty())
            	//Si la cpu no t cartes, li donem
            	this.DealToCpu();
            if(!this.CheckAnimations() && !this._cpu.getHandPlayer().isEmpty() && !this._player.getHandPlayer().isEmpty()) {
            	//Si els dos jugadors tenen cartes, repartim les 4 cartes de la taula inicials, i passem al segent estat
                initDeal();
                this._gameState = gameState.PLAYING;
            }   			 	
			break;
		/* Estat : JUGAR RONDA */
		case PLAYING:
			if (!this.CheckAnimations()) {
				/* Si s la primera m, mirem si hi ha una escombra de ma */
				if (this._deckGame.deckSize() >= 30 && this._player.getHandPlayer().size() == 3 && this._cpu.getHandPlayer().size() == 3 && CheckEscobaHandDealer()) {
					this.InitMsgGame(this._gameLocalActivity.getResources().getText(R.string.msgHAND_BROOM).toString());
				}
				/* Si hi ha cartes a la baralla i els jugadors no en tenen a la m, els hi repartim */
				if(this._deckGame.deckSize() > 0 && this._player.getHandPlayer().isEmpty() && this._cpu.getHandPlayer().isEmpty()) {
					this.DealCards();
					this.OnlyShowOnecardPlayerDeck(); //Netejem les basses per optimitzar recursos
					this.OnlyShowOneCardCpuDeck(); //Netejem les basses per optimitzar recursos
				}
				/* Si ja no tenim cartes, assignem les cartes de taula al ltim jugador que va fer una bassa, i 
				 * passem al segent estat  */
                if(this._deckGame.deckSize() <= 0 && this._player.getHandPlayer().isEmpty() && this._cpu.getHandPlayer().isEmpty()) {
                    if(this._cardsTable.size() > 0)
                    	this.CleanLastCards();
                    this.OnlyShowOnecardPlayerDeck();
            	    this.OnlyShowOneCardCpuDeck();	  
                    this._gameState = gameState.END_ROUND;
                }
                /* Si s el torn de la CPU, amaguem botons, chequejem les cartes que hagi pogut tirar el jugador hum, i fem la jugada */
                if (this._turnGame == turnGame.CPU_PLAYER) {
                	this._gameLocalActivity.PlayButton.setVisible(false);
                	this._gameLocalActivity.ThrowButton.setVisible(false);
                	if(this._playerLostCards.size() <= 0) {
                		//Si no hi ha bassa de taula, la CPU fa la seva jugada
                		if(this._cpu.getHandPlayer().size() > 0 && !this.CheckAnimations()) {
                			this.AIPlayEasy();
                			this._turnGame = turnGame.HUMAN_PLAYER;
                        }
                	} else {
                		//Si hi ha bassa de taula, mostrem un missatge, assignem els punts a la CPU i la recollim
                		this.InitMsgGame(this._gameLocalActivity.getResources().getText(R.string.msgTABLE_TRICK).toString());
                		this._cpu.refreshScorePlayerLostCards(this._playerLostCards, this._cardsTable.size());
                		Iterator<Card> iterator1 = this._playerLostCards.getIterator();
                		while (iterator1.hasNext()) {
                			Card card = (Card)iterator1.next();
                			this._cpu.getCapturedCards().add(card);
	                        card.getSprite().setInverseFlip(true);
	                        card.getSprite().StartTransition(CPU_CAPTUREDCARDS_X, CPU_CAPTUREDCARDS_Y, 0.8F);
	                        this._cardsTable.remove(card);
                		}
                		//Netejem
                		this._playerLostCards.clear();
                	}
                /* Si s el torn del jugador hum, fem visibles els botons */
                } else {
                	if(this._turnGame == turnGame.HUMAN_PLAYER && this._player.getHandPlayer().size() > 0 && !this.CheckAnimations()) {
                		this._gameLocalActivity.PlayButton.setVisible(true);
                		this._gameLocalActivity.ThrowButton.setVisible(true);
                    }
                }    				
			}
			break;			
		/* Estat : FINALITZAR RONDA */
		case END_ROUND:
			if(this._continueGame) {
				//Resetejem puntuacions (pero no la total)
				this._player.ResetScore();
				this._cpu.ResetScore();
				this._continueGame = false;
				this.QuitGameScores(); //Netejem
				if(this._player.getScore().getTotalScore() >= POINTS_TO_WIN || this._cpu.getScore().getTotalScore() >= POINTS_TO_WIN) {
					//Si algn jugador arriba a la puntuaci per guanyar, passem al segent estat de joc
					this._gameState = gameState.GAME_OVER;
					//i mostrem un missatge de final de partida
					this.SetMsgGameOver();
				} else {
					//Si encara no guanya cap jugador, tornem a repartir cartes i iniciem una nova ronda
					this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.score);
					this._gameLocalActivity.mMainScene.unregisterTouchArea(this._gameLocalActivity.score);
					this._gameState = gameState.SHUFFLING;
				}
			} else if(!this._setScoreOnce) {
				//Si no hem mostrat les puntuacions, les mostrem
				this._gameLocalActivity.mMainScene.getLastChild().attachChild(this._gameLocalActivity.score);
				this._gameLocalActivity.mMainScene.registerTouchArea(this._gameLocalActivity.score);
				this._setScoreOnce = true;
				this.CountPoints(); //Contabilitzem les puntuacions
				this.ShowGameScores(); //Mostrem les puntuacions finals de la ronda
				this._gameLocalActivity.PlayButton.setVisible(false);
				this._gameLocalActivity.ThrowButton.setVisible(false);
			}
			break;
		/* Estat : FI DEL JOC */
		case GAME_OVER:
			if(this._continueGame) {
				 this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.tMsgGameOver);
				 this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.trophy);
				 this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.score);
				 this._gameLocalActivity.mMainScene.unregisterTouchArea(this._gameLocalActivity.score);
				 Iterator<Text> iterScore = this._gameLocalActivity.tScore.iterator();				 
				 while (iterScore.hasNext()) {
					 Text scoreText = iterScore.next();
					 this._gameLocalActivity.mMainScene.getLastChild().detachChild(scoreText);
				 }
				 this._gameLocalActivity.tScore.clear();
				 this._continueGame = false;
				 this._player.getScore().resetAll();
				 this._cpu.getScore().resetAll();
				 this._gameState = gameState.SHUFFLING;
			} else {
				//this._gameLocalActivity.finish();
			}
          
			break;
		default:
			break;    		
		}
	}
	
	/**
	 * Controla les accions del event touch associat al bot d'efectuar jugada 
	 */
	protected void onTouchPlayButton() {
		if (this._turnGame == turnGame.HUMAN_PLAYER) {
			//Si s el torn del jugador hum, comprovem la jugada
			CardList selectedCards = SelectedCards(this._cardsTable);
			if (this._player.getCardSelected() != -1 && !selectedCards.isEmpty() && IsValid(this._player.getHandPlayer().get(this._player.getCardSelected()),selectedCards)) {
				this._gameLocalActivity.mGo.play();
				/* Si s escombra, mostrem un missatge */
				if (selectedCards.size() == this._cardsTable.size())
					InitMsgGame(this._gameLocalActivity.getResources().getText(R.string.msgBROOM).toString());
				this._player.refreshScore(this._player.getHandPlayer().get(this._player.getCardSelected()),selectedCards, this._cardsTable.size());
				this._lastPlayerWithValidGame = turnGame.HUMAN_PLAYER;
				this.ErasePlayerCards(selectedCards);
				this._turnGame = turnGame.CPU_PLAYER;
			} else {
				this._gameLocalActivity.mBeep.play();
			}
		}
	}
	
	/**
	 * Controla les accions del event touch associat al bot de tirar carta
	 */
	protected void onTouchThrowButton() {
		if (this._turnGame == turnGame.HUMAN_PLAYER) {
			if (this._player.getCardSelected() != -1) {
				this._gameLocalActivity.mGo.play();
				/* Si s escombra, mostrem un missatge */
				this.CheckPlayerThrowCard(this._player.getHandPlayer().get(this._player.getCardSelected()), this._cardsTable);
				this.ThrowCard(); //Tirem la carta al mig
				this.SortCardsTable(); //Reordenem les cartes de la taula
				//this._gameLocalActivity.mMainScene.registerTouchArea((Scene.ITouchArea)this._player.getHandPlayer().get(this._player.getHandPlayer().size() - 1).getSprite());
				this.DeselectCardsTable();
				this._turnGame = turnGame.CPU_PLAYER;
			} else {
				this._gameLocalActivity.mBeep.play();
			}
		}
	}
	
	/**
	 * Controla les accions del event touch (DOWN) associal al bot de veure les puntuacions
	 */
	protected void onTouchDownInfoButton(){
		if ((!this._setScoreOnce) && (this._gameState == gameState.PLAYING) && (this._deckGame.deckSize() < 40)) {
			this._gameLocalActivity.mMainScene.getLastChild().attachChild(this._gameLocalActivity.score);
			this.ShowGameScores();			
			this._gameLocalActivity.mAmbience.play();						
		}		
	}
	
	/**
	 * Controla les accions del event touch (UP) associal al bot de veure les puntuacions
	 */
	protected void onTouchUpInfoButton(){
		if ((!this._setScoreOnce) && (this._gameState == gameState.PLAYING)) {
			this._gameLocalActivity.mMainScene.getLastChild().detachChild(this._gameLocalActivity.score);
			this.QuitGameScores();	
			this._gameLocalActivity.mAmbience.stop();			
		}		
	}
	
	/**
	 * Controla les accions del event touch associal al panell de les puntuacions
	 */
	protected void onTouchScore(){
		this.initRound();
		this._continueGame = true;
		this._gameLocalActivity.mAmbience.stop();
	}
	
	
	/* Setters i Getters de les propietats privades necessries */
	  
	
}
