package com.sweep15;

import java.util.ArrayList;
import java.util.Iterator;

import com.sweep15.GameElements.Card;
import com.sweep15.GameElements.CardList;

/**
 * Classe que implementa les funcions necessries per obtenir un raonament de joc senzill.
 * 
 * @author Sergi Collado Figueras : PFC - Xarxes de computadors 2011-2012
 *
 */
public class IACpu
{
  private int CalculateSumDiscardCard(Card pCardToDiscard, int pTableRankSum)
  {
	  /* Atorguem una srie de punts en funci de la qualitat de la carta i de la suma de les cartes de la taula */
    int currentPoints = 0;
    /* Si al descartar la carta deixem un valor a la taula per sota de 5 (no es pot fer escombra) */
    if ((pTableRankSum < 4) && (pTableRankSum + pCardToDiscard.getRank().getRankNumber() < 5))
      currentPoints += 40; 
    /* Si la carta s el 7 d'OROS */
    if (pCardToDiscard.getNumber() == 7)
    	return + 1;
    /* Si la carta s un 7 */
    if (pCardToDiscard.getRank().getRankNumber() == 7) //Un 7
    	return + 2;
    /* Si a la taula tenim entre 6 i 14 punts */
    if ((pTableRankSum > 5) && (pTableRankSum < 15))
      {
    	/* Si al descartar fem passar de 15 els punts de la taula (no es pot fer escombra) */
        if (pTableRankSum + pCardToDiscard.getRank().getRankNumber() > 15) {
        	return + 40;
        } else {
        	return + 20;
        } 
      }
    /* Si s un ORO */
    if (pCardToDiscard.getSuit().getSuitNumber() == 1) //OROS
    	return currentPoints + 4;
    return currentPoints + 20; //Per defecte, 20 punts
  }

  private int CalculateValueMove(CardList pCardSelection, int pTableSize)
  {
	  /* Atorguem una srie de punts en funci de la qualitat de la jugada */
		int currentPoints = 0;
		/* Si s una ESCOMBRA */
		if (pCardSelection.size() - 1 == pTableSize)
		  currentPoints += 40;
		/* Recorrem les cartes de la selecci per calcular-ne la puntuaci de la jugada */
		Iterator<Card> iterator = pCardSelection.getIterator();
		while (iterator.hasNext()) {
			Card lCard = (Card)iterator.next();
			/* Si s el 7 d'OROS */
			if (lCard.getNumber() == 7)
				currentPoints += 40;
			/* Si s un 7 */
			if (lCard.getRank().getRankNumber() == 7)
			    currentPoints += 10;
			/* Si s un ORO */
			if (lCard.getSuit().getSuitNumber() == 1) //OROS
			    currentPoints += 4;
			/* Qualsevol carta s'end punts (per afavorir basses amb ms nombre de cartes, si no hi ha cap 'extra') */
			currentPoints += 2;
		}
		return currentPoints;
  }

  public CardList BestChoice(ArrayList<CardList> pListOfPromotedSelections, int pTableSize)
  {
	  /* La millor selecci ser aquella que obtingui millor puntuaci */
	  	CardList lBestSelection = new CardList(); //Inicialitzem per si l'hem de tornar encara que sigui buida
		int bestPuntuation = 0;
		/* Recorrem les seleccions que ens passen */
		Iterator<CardList> iteratorSelections = pListOfPromotedSelections.iterator();
		while (iteratorSelections.hasNext()) {
			CardList lCurrentSelection = new CardList(iteratorSelections.next()); //Copia    	
		    int currentPuntuation = CalculateValueMove(lCurrentSelection, pTableSize);
		    if (currentPuntuation > bestPuntuation) {
		    	/* Si hem calculat una puntuaci millor, ens la quedem */
		    	bestPuntuation = currentPuntuation;
		    	lBestSelection = new CardList(lCurrentSelection);
		    }	    
		}
		return lBestSelection;
  }

  public CardList CheckRejectedCardPlayer(Card pHandCard, CardList pCardsTable)
  {
    CardList lAllCardsToCheck = new CardList();
    lAllCardsToCheck.add(pHandCard);
    return SelectedCardsByCPU(lAllCardsToCheck, pCardsTable);
  }

  public Card DiscardCards(CardList pCardsTable, CardList pHandCards)
  {
	  /* Descartem la carta menys favorable pel contrari */
	  Card lCardToDiscard = (Card)pHandCards.get(0); //Comencem per la primera carta, per si no en tenim ms (ser la que tornariem)
	  /* Si tenim ms de 1 carta a la m, hem de buscar la menys favorable */
	  if (pHandCards.size() > 1) {
		  /* Recorrem les cartes de la m */
		  int bestPoints = 0;
		  Iterator<Card> iterator = pHandCards.getIterator();
		  while (iterator.hasNext()) {
			  /* Calculem la puntuaci per la carta de la m, i ens la quedem si s la millor */
			  Card lCard = iterator.next();
			  int currentPoints = CalculateSumDiscardCard(lCard, pCardsTable.getRankSum());
			  if (currentPoints > bestPoints) {
				  bestPoints = currentPoints;
				  lCardToDiscard = lCard;
			  }
		  }
	  }
	  /* Retornem la millor carta a descartar */
	  return lCardToDiscard;	  
  }

  public CardList SelectedCardsByCPU(CardList pHandCards, CardList pTableCards)
  {
    CardList selection = new CardList();
    ArrayList<CardList> listOfPromotedSelections = new ArrayList<CardList>();
    CardList lTableCards = new CardList(pTableCards); //Copia
    
    /* Busquem totes les combinacions de jugades vlides per cadascuna de les cartes de ma*/
    Iterator<Card> iterator = pHandCards.getIterator();
    while (iterator.hasNext()) {
    	CardList lCurrentSelection = new CardList();
        Card lCard = iterator.next();
        lCurrentSelection.add(lCard);
        findMove(listOfPromotedSelections, lCurrentSelection, lTableCards, 0, lCard.getRank().getRankNumber());
    }
    /* Si hem trobat jugades vlides, busquem la millor */
    if (listOfPromotedSelections.size() > 0)
    	selection = new CardList(BestChoice(listOfPromotedSelections, pTableCards.size()));
    return selection;    
  }

  public void findMove(ArrayList<CardList> pListOfPromotedSelections, CardList pCurrentSelection, CardList pTableCards, int pCardsIndex, int pSelectionRankValue)
  {
    if (pSelectionRankValue == 15) //Si tenim una jugada vlida (suma 15) la afegim a la llista
      pListOfPromotedSelections.add(pCurrentSelection);
    if ((pSelectionRankValue < 15) && (!pTableCards.isEmpty())) {
    	//Si la combinaci no passa de 15, i encara tenim cartes, seguim buscant combinacions
    	for (int index = pCardsIndex; index < pTableCards.size(); index++)
	    {
    		CardList lCurrentSelection = new CardList(pCurrentSelection); //Copia
	        CardList lTableCards = new CardList(pTableCards); //Copia
	        Card lCard = (Card)lTableCards.remove(index);
	        lCurrentSelection.add(lCard);
	        findMove(pListOfPromotedSelections, lCurrentSelection, lTableCards, index, pSelectionRankValue + lCard.getRank().getRankNumber());
	    }
    }
  }  

}