package com.sweep15;

import java.util.Locale;

import javax.microedition.khronos.opengles.GL10;

import org.anddev.andengine.engine.Engine;
import org.anddev.andengine.engine.camera.Camera;
import org.anddev.andengine.engine.options.EngineOptions;
import org.anddev.andengine.engine.options.EngineOptions.ScreenOrientation;
import org.anddev.andengine.engine.options.resolutionpolicy.RatioResolutionPolicy;
import org.anddev.andengine.entity.scene.Scene;
import org.anddev.andengine.entity.scene.background.SpriteBackground;
import org.anddev.andengine.entity.scene.menu.MenuScene;
import org.anddev.andengine.entity.scene.menu.MenuScene.IOnMenuItemClickListener;
import org.anddev.andengine.entity.scene.menu.item.IMenuItem;
import org.anddev.andengine.entity.scene.menu.item.TextMenuItem;
import org.anddev.andengine.entity.scene.menu.item.decorator.ColorMenuItemDecorator;
import org.anddev.andengine.entity.sprite.Sprite;
import org.anddev.andengine.opengl.font.Font;
import org.anddev.andengine.opengl.font.FontFactory;
import org.anddev.andengine.opengl.texture.TextureOptions;
import org.anddev.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlas;
import org.anddev.andengine.opengl.texture.atlas.bitmap.BitmapTextureAtlasTextureRegionFactory;
import org.anddev.andengine.opengl.texture.region.TextureRegion;
import org.anddev.andengine.ui.activity.BaseGameActivity;

import com.sweep15.AndEextras.PositionalMenuAnimator;

import android.content.Intent;
import android.content.res.Configuration;
import android.graphics.Color;
import android.view.KeyEvent;

/**
 * Classe que implementa l'activitat principal de l'aplicaci, a partir de la qual es generar 
 * el men principal i la crida a la resta d'activitats.
 * 
 * @author Sergi Collado Figueras : PFC - Xarxes de computadors 2011-2012
 *
 */
public class MainActivity extends BaseGameActivity implements IOnMenuItemClickListener {
	
	/* Constants */

	public static final int CAMERA_WIDTH = 720;
	public static final int CAMERA_HEIGHT = 480;

	//Men principal
	protected static final int MENU_PLAY = 0;
	protected static final int MENU_LANGUAGE = MENU_PLAY + 1;
	protected static final int MENU_HOWTOPLAY = MENU_LANGUAGE + 1;
	protected static final int MENU_QUIT = MENU_HOWTOPLAY + 1;
	
	//Submen Jugar
	protected static final int MENU_VS_CPU_EASY = MENU_QUIT + 1;
	protected static final int MENU_VS_CPU_HARD = MENU_VS_CPU_EASY + 1;
	protected static final int MENU_VS_ONLINE = MENU_VS_CPU_HARD + 1;
	
	//Submen Idioma
	protected static final int MENU_LANG_CAT = MENU_VS_ONLINE + 1;
	protected static final int MENU_LANG_ESP = MENU_LANG_CAT + 1;
	protected static final int MENU_LANG_ENG = MENU_LANG_ESP + 1;

	/* Propietats */
	
	protected Camera mCamera;
	
	protected MenuScene mMenuScene;
	protected MenuScene mPlayMenuScene;
	protected MenuScene mLanguageMenuScene;
	protected Scene mMainScene;
	protected Scene mGameFieldScene;

	private BitmapTextureAtlas mBitmapTextureAtlasTitle;
	private TextureRegion mTitleBackgroundTextureRegion;
	
	private BitmapTextureAtlas mFontTexture;

	private Font mFont;
	
	private Locale locCat;
	private Locale locEsp;
	private Locale locEng;

	public static MainActivity gameMainActivity;
	public static MainActivity getInstance(){return gameMainActivity;}

	/* Sobrecrrega de mtodes necessaris per a controlar l'activitat */
	
	@Override
	public Engine onLoadEngine() {
		this.mCamera = new Camera(0, 0, CAMERA_WIDTH, CAMERA_HEIGHT);
		gameMainActivity = this;
		return new Engine(new EngineOptions(true, ScreenOrientation.LANDSCAPE, new RatioResolutionPolicy(CAMERA_WIDTH, CAMERA_HEIGHT), this.mCamera));		
	}

	@Override
	public void onLoadResources() {
		/* Carrega les textures per les fonts del men */
		this.mFontTexture = new BitmapTextureAtlas(256, 256, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		FontFactory.setAssetBasePath("font/");
		this.mFont = FontFactory.createFromAsset(this.mFontTexture, this, "Matiz.ttf", 48, true, Color.WHITE);	
		this.mEngine.getTextureManager().loadTextures(this.mFontTexture);
		this.getFontManager().loadFonts(this.mFont);
				
		/* Carrega el fons del men principal */
        this.mBitmapTextureAtlasTitle = new BitmapTextureAtlas(1024 ,512, TextureOptions.BILINEAR_PREMULTIPLYALPHA);
		BitmapTextureAtlasTextureRegionFactory.setAssetBasePath("gfx/");
		this.mTitleBackgroundTextureRegion = BitmapTextureAtlasTextureRegionFactory.createFromAsset(this.mBitmapTextureAtlasTitle, this, "Menu-Background.jpg", 0, 0);				        

        this.mEngine.getTextureManager().loadTexture(this.mBitmapTextureAtlasTitle);       
                
        locCat = new Locale("cat"); //Per defecte no el trobar, i llegira de /values
        locEng = new Locale("en");
        locEsp = new Locale("es");
        //El catal caldr triar-lo del men
        if (Locale.getDefault().getLanguage() != "en") {
        	Locale.setDefault(locEsp);
        } else {
        	Locale.setDefault(locEng);
        }
        //Locale.setDefault(locCat);
        Configuration config = new Configuration();
        config.locale = Locale.getDefault();
        getBaseContext().getResources().updateConfiguration(config, null);
	}

	@Override
	public Scene onLoadScene() {
		this.mMenuScene = this.createMenuScene();
		//this.mPlayMenuScene = this.createPlayMenuScene();
		this.mMainScene = new Scene();
	    this.mMainScene.setChildScene(this.mMenuScene, false, true, true);
	    this.mMainScene.setBackground(new SpriteBackground(new Sprite(0.0F, 0.0F, this.mTitleBackgroundTextureRegion)));
	    return this.mMainScene;
	}

	@Override
	public void onLoadComplete() {

	}

	@Override
	public boolean onKeyDown(final int pKeyCode, final KeyEvent pEvent) {
		if(pKeyCode == KeyEvent.KEYCODE_MENU && pEvent.getAction() == KeyEvent.ACTION_DOWN) {
			if(this.mMenuScene.hasChildScene()) {
				/* Remove the menu and reset it. */
				//this.mPlayMenuScene.back();
				this.mMenuScene.clearChildScene();
			} else {
				/* Attach the menu. */
				//this.mEngine.setScene(this.mMenuScene);
				//this.mMenuScene.setChildScene(this.mPlayMenuScene, false, true, true);
			}
			return true;
		} else {
			return super.onKeyDown(pKeyCode, pEvent);
		}
	}

	@Override
	public boolean onMenuItemClicked(final MenuScene pMenuScene, final IMenuItem pMenuItem, final float pMenuItemLocalX, final float pMenuItemLocalY) {
		switch(pMenuItem.getID()) {
		
			/* MENU PRINCIPAL */
			case MENU_PLAY:
				/* Mostrem el submen de selecci de joc */
				this.mPlayMenuScene = this.createPlayMenuScene();
				this.mMenuScene.setChildSceneModal(this.mPlayMenuScene);				
				return true;
			case MENU_LANGUAGE:
				/* Mostrem el men d'idioma */
				this.mLanguageMenuScene = this.createLanguageMenuScene();
				this.mMenuScene.setChildSceneModal(this.mLanguageMenuScene);
				return true;
			case MENU_HOWTOPLAY:
				/* Mostrem les instruccions */
				Intent howToPlayIntent = new Intent();
			    howToPlayIntent.setClass(getApplicationContext(), HowToPlayActivity.class);
			    startActivity(howToPlayIntent);			    
				return true;
			case MENU_QUIT:
				/* Finalitzem */
				//this.finish();
				System.exit(0);
				return true;
				
			/* SUBMENU TIPUS DE JOC */
			case MENU_VS_CPU_EASY:
				/* Joc contra CPU - Nivell fcil */
				Intent playIntent = new Intent();
			    playIntent.setClass(getApplicationContext(), Sweep15GameLocalActivity.class);
			    startActivity(playIntent);						
				return true;
			case MENU_VS_CPU_HARD:
				//TODO Implementar joc CPU ms complex
				return true;
			case MENU_VS_ONLINE:
				//TODO Implementar joc Online
				return true;
				
			/* SUBMENU IDIOMA */
			case MENU_LANG_CAT: {
				Locale.setDefault(locCat);		        		        
		        Configuration config = new Configuration();
		        config.locale = Locale.getDefault();
		        getBaseContext().getResources().updateConfiguration(config, null);
		        this.mMainScene.clearChildScene();
			    this.mEngine.setScene(onLoadScene());		        
		        return true;
			}				
			case MENU_LANG_ESP: {
				Locale.setDefault(locEsp);		        		        
		        Configuration config = new Configuration();
		        config.locale = Locale.getDefault();
		        getBaseContext().getResources().updateConfiguration(config, null);
		        this.mMainScene.clearChildScene();
			    this.mEngine.setScene(onLoadScene());		        
		        return true;
			}				
			case MENU_LANG_ENG: {
				Locale.setDefault(locEng);		        		        
		        Configuration config = new Configuration();
		        config.locale = Locale.getDefault();
		        getBaseContext().getResources().updateConfiguration(config, null);
		        this.mMainScene.clearChildScene();
			    this.mEngine.setScene(onLoadScene());		        
		        return true;
			}
												
			default:
				return false;
		}
	}

	/* Mtodes */
	
	/**
	 * Crea la escena corresponent al men principal
	 */
	protected MenuScene createMenuScene() {
		final MenuScene menuScene = new MenuScene(this.mCamera);

		final TextMenuItem playTextMenuItem = new TextMenuItem(MENU_PLAY, this.mFont, getResources().getText(R.string.mnuMAIN_PLAY).toString()) {
			public void onSelected() {setScale(1.3F);}
			public void onUnselected() {setScale(1.0F);}
		};
		final IMenuItem playMenuItem = new ColorMenuItemDecorator(playTextMenuItem, 1.0f,0.0f,0.0f, 0.8f,0.8f,0.0f);
		playMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		menuScene.addMenuItem(playMenuItem);
	
		final TextMenuItem optionsTextMenuItem = new TextMenuItem(MENU_LANGUAGE, this.mFont, getResources().getText(R.string.mnuMAIN_LANGUAGE).toString()) {
			public void onSelected() {setScale(1.3F);}
			public void onUnselected() {setScale(1.0F);}
		};
		final IMenuItem optionsMenuItem = new ColorMenuItemDecorator(optionsTextMenuItem, 1.0f,0.0f,0.0f, 0.8f,0.8f,0.0f);
		optionsMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		menuScene.addMenuItem(optionsMenuItem);
		
		final TextMenuItem howToPlayTextMenuItem = new TextMenuItem(MENU_HOWTOPLAY, this.mFont, getResources().getText(R.string.mnuMAIN_HOWTOPLAY).toString()) {
			public void onSelected() {setScale(1.3F);}
			public void onUnselected() {setScale(1.0F);}
		};
		final IMenuItem howToPlayMenuItem = new ColorMenuItemDecorator(howToPlayTextMenuItem, 1.0f,0.0f,0.0f, 0.8f,0.8f,0.0f);
		howToPlayMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		menuScene.addMenuItem(howToPlayMenuItem);
		
		final TextMenuItem quitTextMenuItem = new TextMenuItem(MENU_QUIT, this.mFont, getResources().getText(R.string.mnuMAIN_QUIT).toString()) {
			public void onSelected() {setScale(1.3F);}
			public void onUnselected() {setScale(1.0F);}
		};
		final IMenuItem quitMenuItem = new ColorMenuItemDecorator(quitTextMenuItem, 1.0f,0.0f,0.0f, 0.8f,0.8f,0.0f);
		quitMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		menuScene.addMenuItem(quitMenuItem);

		menuScene.setMenuAnimator(new PositionalMenuAnimator(320,210,10));
		menuScene.buildAnimations();
		
		menuScene.setBackgroundEnabled(false);
		
		menuScene.setOnMenuItemClickListener(this);
		return menuScene;
	}
	
	/**
	 * Crea la escena corresponent al submen de selecci de joc
	 */
	protected MenuScene createPlayMenuScene() {
		final MenuScene subMenuScene = new MenuScene(this.mCamera);

		final TextMenuItem vsCpuEasyTextMenuItem = new TextMenuItem(MENU_VS_CPU_EASY, this.mFont, getResources().getText(R.string.mnuPLAY_VS_CPU_EASY).toString()) {
			public void onSelected() {setScale(1.3F);}
			public void onUnselected() {setScale(1.0F);}
		};
		final IMenuItem vsCpuEasyMenuItem = new ColorMenuItemDecorator(vsCpuEasyTextMenuItem, 1.0f,0.0f,0.0f, 0.8f,0.8f,0.0f);
		vsCpuEasyMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		subMenuScene.addMenuItem(vsCpuEasyMenuItem);
	
		final TextMenuItem vsCpuHardTextMenuItem = new TextMenuItem(MENU_VS_CPU_HARD, this.mFont, getResources().getText(R.string.mnuPLAY_VS_CPU_HARD).toString()) {
			public void onSelected() {setScale(1.3F);}
			public void onUnselected() {setScale(1.0F);}
		};
		final IMenuItem vsCpuHardMenuItem = new ColorMenuItemDecorator(vsCpuHardTextMenuItem, 1.0f,0.0f,0.0f, 0.8f,0.8f,0.0f);
		vsCpuHardMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		subMenuScene.addMenuItem(vsCpuHardMenuItem);
		
		final TextMenuItem vsOnlinePlayTextMenuItem = new TextMenuItem(MENU_VS_ONLINE, this.mFont, getResources().getText(R.string.mnuPLAY_VS_ONLINE).toString()) {
			public void onSelected() {setScale(1.3F);}
			public void onUnselected() {setScale(1.0F);}
		};
		final IMenuItem vsOnlinePlayMenuItem = new ColorMenuItemDecorator(vsOnlinePlayTextMenuItem, 1.0f,0.0f,0.0f, 0.8f,0.8f,0.0f);
		vsOnlinePlayMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		subMenuScene.addMenuItem(vsOnlinePlayMenuItem);			

		subMenuScene.setMenuAnimator(new PositionalMenuAnimator(300,270,10));
		subMenuScene.buildAnimations();
		
		subMenuScene.setBackgroundEnabled(false);
		
		subMenuScene.setOnMenuItemClickListener(this);
		return subMenuScene;
	}
	
	/**
	 * Crea la escena corresponent al submen de Idioma
	 */
	protected MenuScene createLanguageMenuScene() {
		final MenuScene subMenuScene = new MenuScene(this.mCamera);

		final TextMenuItem langCatTextMenuItem = new TextMenuItem(MENU_LANG_CAT, this.mFont, "Cat") {
			public void onSelected() {setScale(1.3F);}
			public void onUnselected() {setScale(1.0F);}
		};
		final IMenuItem langCatMenuItem = new ColorMenuItemDecorator(langCatTextMenuItem, 1.0f,0.0f,0.0f, 0.8f,0.8f,0.0f);
		langCatMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		subMenuScene.addMenuItem(langCatMenuItem);
	
		final TextMenuItem langEspTextMenuItem = new TextMenuItem(MENU_LANG_ESP, this.mFont, "Esp") {
			public void onSelected() {setScale(1.3F);}
			public void onUnselected() {setScale(1.0F);}
		};
		final IMenuItem langEspMenuItem = new ColorMenuItemDecorator(langEspTextMenuItem, 1.0f,0.0f,0.0f, 0.8f,0.8f,0.0f);
		langEspMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		subMenuScene.addMenuItem(langEspMenuItem);
		
		final TextMenuItem langEngTextMenuItem = new TextMenuItem(MENU_LANG_ENG, this.mFont, "Eng") {
			public void onSelected() {setScale(1.3F);}
			public void onUnselected() {setScale(1.0F);}
		};
		final IMenuItem langEngMenuItem = new ColorMenuItemDecorator(langEngTextMenuItem, 1.0f,0.0f,0.0f, 0.8f,0.8f,0.0f);
		langEngMenuItem.setBlendFunction(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
		subMenuScene.addMenuItem(langEngMenuItem);			

		subMenuScene.setMenuAnimator(new PositionalMenuAnimator(400,270,10));
		subMenuScene.buildAnimations();
		
		subMenuScene.setBackgroundEnabled(false);
		
		subMenuScene.setOnMenuItemClickListener(this);
		return subMenuScene;
	}
}