package com.sweep15;

import java.util.Iterator;

import com.sweep15.GameElements.Card;
import com.sweep15.GameElements.CardList;

/**
 * Classe que implementa a un jugador
 * 
 * @author Sergi Collado Figueras : PFC - Xarxes de computadors 2011-2012
 *
 */
public class Player
{	
	public static int NUM_HAND_CARDS = 3; //3 cartes a la m
	
	private int cardSelected; //til noms quan s hum
	private CardList handPlayer; //Cartes en la m del jugador
	private CardList capturedCards; //Cartes de les basses que ha anat aconseguint
	private String playerName;
	private Score score;
	
	protected boolean isDealer;

	/* Constructor */
		
	public Player(String pPlayerName) {
		this.setPlayerName(pPlayerName);
		this.setHandPlayer(new CardList(NUM_HAND_CARDS)); 
		this.setCapturedCards(new CardList()); //Cartes de les basses
		this.setCardSelected(-1); //Sense selecci = -1
		this.setScore(new Score()); //Puntuaci del jugador
		this.isDealer = false; //Si s el repartidor
	}

	/**
	 * Actualitza la puntuaci del jugador, tenint en compte la carta de m i una llista de cartes (les de la taula
	 * que formen part de la jugada)
	 * 
	 * @param pHandCardToEval : Carta de la m
	 * @param pTableCardsToEval : Cartes de taula de la jugada
	 * @param pTableSize : Nombre total de cartes que hi ha a la taula
	 */
	public void refreshScore(Card pHandCardToEval, CardList pTableCardsToEval, int pTableSize) {
		/* ESCOMBRA */
		if (pTableCardsToEval.size() == pTableSize)
			this.getScore().setBroomCount(this.getScore().getBroomCount() + 1);
		/* Num. cartes */
		this.getScore().setCardsCount(this.getScore().getCardsCount() + 1 + pTableCardsToEval.size()); //S'ha de contar la carta de m
      
		CardList lCardList = new CardList(pTableCardsToEval);
		lCardList.add(pHandCardToEval); //Les recorrem totes, la de m inclosa
      
		Iterator<Card> iterator = lCardList.getIterator();
		while (iterator.hasNext()) {
			Card lCard = iterator.next();    	  
			if (lCard.getNumber() == 7) //7 de OROS
				this.getScore().setSevenGold(this.getScore().getSevenGold() + 1);
			if (lCard.getRank().getRankNumber() == 7) //Nombre de 7
				this.getScore().setSevenCount(this.getScore().getSevenCount() + 1);	      
			if (lCard.getSuit().getSuitNumber() == 1) //Oros
				this.getScore().setGoldCount(this.getScore().getGoldCount() + 1);
		}
	}

	/**
	 * Actualitza la puntuaci de les escombres obtingudes quan es dona que el repartidor 
	 * obt una escombra de m.
	 * 
	 * @param pDealerHand : Cartes de la taula de m
	 */
	public void refreshScoreEscobaHandDealer(CardList pDealerHand) {
		this.getScore().setBroomCount(this.getScore().getBroomCount() + 1);
		/* Avaluem la resta de punts que atorgui la jugada */
		refreshScoreLastCards(pDealerHand);
	}
	
	/**
	 * Actualitza la puntuaci de les jugades (excepte d'escombra) obtingudes en un conjunt de 
	 * cartes (cartes de escombra de m, o ltimes cartes de la m).
	 * 
	 * @param pLastHand : Cartes de les que avaluar-ne els punts
	 */
	public void refreshScoreLastCards(CardList pLastHand) {
		/* Num. cartes */
		this.getScore().setCardsCount(this.getScore().getCardsCount() + pLastHand.size());		
     
		CardList lCardList = new CardList(pLastHand);
		Iterator<Card> iterator = lCardList.getIterator();
		while (iterator.hasNext()) {
			Card lCard = iterator.next();    	  
			if (lCard.getNumber() == 7) //7 de OROS
				this.getScore().setSevenGold(this.getScore().getSevenGold() + 1);
			if (lCard.getRank().getRankNumber() == 7) //Nombre de 7
				this.getScore().setSevenCount(this.getScore().getSevenCount() + 1);		      
			if (lCard.getSuit().getSuitNumber() == 1) //Oros
				this.getScore().setGoldCount(this.getScore().getGoldCount() + 1);
		}
	}

	public void refreshScorePlayerLostCards(CardList paramArrayList, int paramInt) {
//		if (paramArrayList.size() == paramInt) {
//			int[] arrayOfInt = this.stats;
//			arrayOfInt[1] = (1 + arrayOfInt[1]);
//		}
//		refreshScoreLastCards(paramArrayList);
	}

	public void ResetScore()
	{
		this.getScore().reset();
	}

	/**
	 * Funci per saber si hi ha alguna animaci en curs a les cartes de la m del jugador
	 * @return true en cas afirmatiu
	 */
	public boolean isAnimating() {
		return this.getHandPlayer().isRunningAnimation();        
	}
  
	/* Setters i Getters de les propietats privades necessries */
  
	public void setCardSelected(int cardSelected) {
		this.cardSelected = cardSelected;
	}
	
	public int getCardSelected() {
		return cardSelected;
	}

	public void setHandPlayer(CardList handPlayer) {
		this.handPlayer = handPlayer;
	}

	public CardList getHandPlayer() {
		return handPlayer;
	}

	public void setPlayerName(String playerName) {
		this.playerName = playerName;
	}

	public String getPlayerName() {
		return playerName;
	}

	public void setCapturedCards(CardList capturedCards) {
		this.capturedCards = capturedCards;
	}

	public CardList getCapturedCards() {
		return capturedCards;
	}

	public void setScore(Score score) {
		this.score = score;
	}

	public Score getScore() {
		return score;
	}
}
